import Orange.bio.gene

#matching targets are NCBI gene IDs
targets = Orange.bio.gene.NCBIGeneInfo("Homo sapiens").keys()

gm = Orange.bio.gene.GMNCBI("9606")
gm.set_targets(targets)

for gene in [ "cct7", "pls1", "gdi1", "nfkb2", "dlg7" ]:
    print 'Gene', gene, 'is NCBI gene', gm.umatch(gene)



import Orange
from Orange.bio import dicty, geneset, gsea, gene

dbc = dicty.DictyExpress()
data = dbc.get_data(sample='pkaC-', time="8")

#select the first chip (the first attribute)
data = data.translate([data.domain.attributes[0]], True)

matcher = gene.matcher([[gene.GMKEGG("dicty"), gene.GMDicty()]])
genesets =  geneset.collections((("KEGG",), "dicty"))

res = gsea.direct(data, matcher=matcher, min_part=0.05, 
    gene_sets=genesets)

print "%-40s %6s %6s %6s %7s" % ("LABEL", "NES", "P-VAL", "SIZE", "MATCHED")
for name,resu in sorted(res.items()[:10], key=lambda x: x[1]["p"]): 
    print "%-40s %6.3f %6.3f %6d %7d" % (name.name[:35], resu["nes"],
        resu["p"], resu["size"], resu["matched_size"]) 




"""<name>BioMart</name>
<description>Query BioMart service</description>
<contact>Ales Erjavec (ales.erjavec(@at@)fri.uni-lj.si</contact>
<priority>2010</priority>
<icon>icons/BioMart.svg</icon>
"""

from __future__ import absolute_import

import sys, os
import traceback
import warnings
import socket
from collections import defaultdict
import itertools

import Orange

from Orange.OrangeWidgets import OWConcurrent
from Orange.OrangeWidgets.OWWidget import *

from .. import obiBioMart
from ..obiBioMart import *

NAME = "BioMart"
DESCRIPTION = "Query BioMart service"
ICON = "icons/BioMart.svg"
PRIORITY = 2010

INPUTS = [("Input ids", Orange.data.Table, "setData")]
OUTPUTS = [("Example Table", Orange.data.Table)]

REPLACES = ["_bioinformatics.widgets.OWBioMart.OWBioMart"]


socket.setdefaulttimeout(60)


def is_hidden(tree):
    return getattr(tree, "hidden", "false") != "false" or getattr(tree, "hideDisplay", "false") != "false"

class Control(object):
    """ Base mixin class for query GUI widgets 
    """
    def __init__(self, tree, dataset, master):
        """ 
            :param tree: configuration element
            :type tree: obiBioMart.ConfigurationNode
            :param dataset: dataset
            :type dataset: obiBioMart.BioMartDataset
            :param master: main widget
            :type master: OWBioMart
            
        """
        self.tree = tree
        self.dataset = dataset
        self.master = master
        self.subControls = []
        
        if isinstance(self, QObject):
            self.setObjectName(tree.internalName)
            if hasattr(tree, "description"):
                self.setToolTip(tree.description)
        
    def addSubControl(self, tree, control):
        self.subControls.append((tree, control))
        
    def registerDelayedCall(self, call):
        self.master.registerDelayedCall(call)
        
    def pushAction(self, action):
        self.master.pushAction(action)
        
    def query(self):
        return itertools.chain(*[ctrl.query() for _, ctrl in self.subControls])
    
    def setControlValue(self, name, value):
        if "." in name:
            name, rest = name.split(".", 1)
            controls = dict([(tree.internalName, control) for tree, control in self.subControls])
            ctrl = controls.get("name", None)
            if ctrl:
                ctrl.setControlValue(rest, value)
            
        
class UnknownFilter(QWidget, Control):
    def __init__(self, tree, dataset, master, parent=None):
        QWidget.__init__(self, parent)
        Control.__init__(self, tree, dataset, master)
    
class TextFieldFilter(QLineEdit, Control):
    """ A single edit line filter
    """
    def __init__(self, tree, dataset, master, parent=None):
        QLineEdit.__init__(self, parent)
        Control.__init__(self, tree, dataset, master)
        
        if hasattr(tree, "regexp"):
            self.setValidator(QRegExpValidator(QRegExp(tree.regexp), self))
            
        if hasattr(tree, "defaultValue"):
            self.setText(tree.defaultValue)
        
        
    def get_filter(self):
        return self.tree.internalName, str(self.text())
    
    def query(self):
        return [("Filter", self.tree, str(self.text()))]
    
    def setControlValue(self, name, value):
        self.setText(value)
        
        
class IdListFilter(QWidget, Control):
    """ Multiple ids filter
    """
    def __init__(self, tree, dataset, master, parent=None):
        QWidget.__init__(self, parent)
        Control.__init__(self, tree, dataset, master)
        self.tree = tree
        self.dataset = dataset
        self.master = master
        self.setObjectName(tree.internalName)
        
        self.setLayout(QGridLayout())
        self.textWidget = QPlainTextEdit() #TODO: Subclass to recieve drop events from item model views
        self.layout().addWidget(self.textWidget, 0, 0, 1, 1)
        
    def value(self):
        """ Return filter value for use in a query
        """
        return str(self.textWidget.toPlainText()).split()
    
    def get_filter(self):
        return self.tree.internalName, self.value()
    
    def query(self):
        return [("Filter", self.tree, self.value())]
    
    def setControlValue(self, name, value):
        if type(value) == list:
            value = "\n".join(value)
        self.textWidget.setPlainText(value)
            
    
class RadioBooleanFilter(QWidget, Control):
    """ Boolean filter (Only/Exclude)
    """
    def __init__(self, tree, dataset, master, parent=None):
        QWidget.__init__(self, parent)
        Control.__init__(self, tree, dataset, master)
        
        self.setLayout(QVBoxLayout())
        self.buttonGroup = QButtonGroup(self)
        self.values = []
        for i, option in enumerate(tree.subelements_top("Option")):
            rb = QRadioButton(option.displayName, self)
            self.buttonGroup.addButton(rb)
            self.buttonGroup.setId(rb, i)
            self.layout().addWidget(rb)
            self.values.append(option.value)
        self.buttonGroup.button(0).setChecked(True)
            
    def value(self):
        return {"excluded": "%i" % self.buttonGroup.checkedId()}
    
    def get_filter(self):
        return self.tree.internalName, self.value()
    
    def query(self):
        return [("Filter", self.tree, self.value())]
    
    def setControlValue(self, name, value):
        for i, v in enumerate(self.values):
            if v == value:
                button = self.buttonGroup.button(i)
                button.setChecked(True)
                break
                
                
class DropDownFilter(QComboBox, Control):
    """ List menu filter
    """
    def __init__(self, tree, dataset, master, parent=None):
        QComboBox.__init__(self, parent)
        Control.__init__(self, tree, dataset, master)
        
        self.options = []            
        self.selectedIndex = 0
        
        if getattr(tree, "graph", "0") == "1":
            self.setOptions(tree.subelements("Option"))
        else:
            self.setOptions(tree.subelements_top("Option"))
                
        self.connect(self, SIGNAL("currentIndexChanged(int)"), self.onIndexChange)
        
    def setOptions(self, options):
        self.options = []
        self.blockSignals(True)
        self.clear()
        for option in options:
            self.addItem(option.displayName)
            self.options.append(option)
        self.selectedIndex = 0
        self.blockSignals(False)
        self.registerDelayedCall(lambda: self.onIndexChange(0))
        
    def onIndexChange(self, index):
        if self.options:
            option = self.options[index]
            self.selectedIndex = index
            pushActions = option.subelements_top("PushAction")
            for action in pushActions:
                self.master.pushAction(action)
            
    def value(self):
        option = self.options[self.selectedIndex]
        return option.value
    
    def query(self):
        return [("Filter", self.tree, self.value())]
    
    def setControlValue(self, name, value):
        for i, option in enumerate(self.options):
            if option.value == value:
                self.setCurrentIndex(i)
                
    
#class GraphDropDownFilter(QComboBox, Control):
#    """ Tree menu filter
#    """
#    def __init__(self, tree, dataset, master, parent=None):
#        QComboBox.__init__(self, parent)
#        Control.__init__(self, tree, dataset, master)
#        
#        self.options = []
#        self.selectedIndex = 0
#
#        self.installEventFilter(self)
#        
#        self.setOptions(tree.subelements_top("Option"))
#                
##        self.connect(self, SIGNAL("currentIndexChanged(int)"), self.onIndexChange)
#        
#    def setOptions(self, options):
#        self.options = list(options)
#        self.clear()
#        self.addItem(self.options[0].displayName)
#        self.value = self.options[0].value
#        
#    def eventFilter(self, obj, event):
#        if event.type() == QEvent.MouseButtonPress:
#            self.showMenu()
#            return True
#        elif event.type() == QEvent.MouseButtonRelease:
#            return True
#        return False
#    
#    def showPopup(self):
#        return
#    
#    def closePopup(self):
#        return
#    
#    def buildMenu(self):
#        menu = QMenu(self)
#        
#        def addOption(menu, option):
#            suboptions = list(option.subelements_top("Option"))
#            if suboptions:
#                submenu = menu.addMenu(option.displayName)
#                self.connect(submenu, SIGNAL("hovered(QAction)"), self.updateLastHovered)
#                for op in suboptions:
#                    addOption(submenu, op)
#            else:
#                action = menu.addAction(option.displayName)
#                action._value = option.value
#                
#        for option in self.options:
#            addOption(menu, option)
#        
#        self.connect(menu, SIGNAL("hovered(QAction)"), self.updateLastHovered)
#        self.connect(menu, SIGNAL("triggered(QAction)"), lambda action: menu.close())
#        return menu 
#    
#    def updateLastHovered(self, action):
#        self.lastHovered = str(action.text())
#        print self.lastHovered
#    def showMenu(self):
#        menu = self.buildMenu()
#        action = menu.exec_(self.mapToGlobal(QPoint(0, 0)))
#        name = str(action.text())
#        self._value = value = action._value
#        self.setItemText(0, name)
#                    
#    def query(self):
#        return [("Filter", self.tree, self._value)]
    
    
def rows(index_list):
    return map(QModelIndex.row, index_list)


class MultiSelectListFilter(QListView, Control):
    """ Menu list filter with multiple selection
    """
    def __init__(self, tree, dataset, master, parent=None):
        QComboBox.__init__(self, parent)
        Control.__init__(self, tree, dataset, master)
        
        self.setSelectionMode(QListView.ExtendedSelection)
        model = QStringListModel(self)
        self.setModel(model)
        self.setOptions(tree.subelements_top("Option"))
        
        
    def setOptions(self, options):
        self.options = []
        for option in options:
            self.options.append(option)
        self.model().setStringList([option.displayName for option in self.options])
                
    def value(self):
        value = []
        for index in rows(self.selectedIndexes()):
            value.append(self.options[index].value)
        return ",".join(value)
    
    def query(self):
        return [("Filter", self.tree, self.value())]
    
    def setControlValue(self, name, value):
        if type(value) == str:
            value = value.split(",")
        selection = self.selectionModel()
        for i, option in enumerate(self.options):
            if option.value in values:
                selection.select(self.model().index(i), QItemSelectionModel.Select)
            
    
class DropDownRadioBooleanFilter(QWidget, Control):
    """ Container for multiple boolean filters
    """
    def __init__(self, tree, dataset, master, parent=None):
        QWidget.__init__(self, parent)
        Control.__init__(self, tree, dataset, master)
        
        self.setLayout(QHBoxLayout())
        self.cb = QComboBox(self)
        
        self.layout().addWidget(self.cb)
        
        rblayout = QVBoxLayout()
        self.radioButtons = [QRadioButton("Only", self),
                             QRadioButton("Excluded", self)
                             ]
        
        for b in self.radioButtons:
            rblayout.addWidget(b)
            
        self.radioButtons[0].setChecked(True)
            
        self.layout().addLayout(rblayout)
        
        self.options = []
        
        self.setOptions(tree.subelements_top("Option"))
    
    def setOptions(self, options):
        self.cb.clear()
        self.options = []
        for option in options:
            self.cb.addItem(option.displayName)
            self.options.append(option)
            
        for op, rb in zip(self.options[0].subelements_top("Option"),
                          self.radioButtons):
            rb.setText(op.displayName)
            rb.setChecked(getattr(op, "default", "false") == "true")
            

    def value(self):
        return {"excluded": "0" if self.radioButtons[0].isChecked() else "1"}
    
    def query(self):
        filter = self.options[self.cb.currentIndex()]
        filter = FilterDescription(self.tree.registry, "FilterDescription", filter.attributes, filter.children)
        return [("Filter", filter, self.value())] #"only" if self.radioButtons[0].isChecked() else "excluded")]
    
    def setControlValue(self, name, value):
        for i, option in enumerate(self.options):
            if option.internalName == name:
                self.cb.setCurrentIndex(i)
                if value == "Only":
                    self.radioButtons[0].setChecked(True)


class DropDownIdListFilter(QWidget, Control):
    """ Container for multiple id list filters
    """
    def __init__(self, tree, dataset, master, parent=None):
        QWidget.__init__(self, parent)
        Control.__init__(self, tree, dataset, master)
        
        self.setLayout(QVBoxLayout())
        self.setContentsMargins(0, 0, 0, 0)
        self.cb = QComboBox()
        self.idsEdit = QPlainTextEdit()
#        self.browseButton = QPushButton("Browse ...")
        
        self.layout().addWidget(self.cb)
        self.layout().addWidget(self.idsEdit)
#        self.layout().addWidget(self.browseButton)
#        self.layout().setAlignment(self.browseButton, Qt.AlignRight)
        
        self.options = []
        self.setOptions(tree.subelements_top("Option"))
        
    def setOptions(self, options):
        self.cb.clear()
        self.options = []
        for option in options:
            self.cb.addItem(option.displayName)
            self.options.append(option)
            
        
    def value(self):
        return str(self.idsEdit.toPlainText()).split()
    
    def query(self):
        filter = self.options[self.cb.currentIndex()]
        filter = FilterDescription(self.tree.registry, "FilterDescription", filter.attributes, filter.children)
        return [("Filter", filter, self.value())]
    
    def setControlValue(self, name, value):
        if type(value) == list:
            value = "\n".join(value)
            
        for i, op in enumerate(self.options):
            if name == op.internalName:
                self.cb.setCurrentIndex(i)
                self.idsEdit.setPlainText(value)
    
        
class CollectionWidget(QGroupBox, Control):
    NO_FLAGS = 0
    FILTER_FLAG = 1
    ATTRIBUTE_FLAG = 2
    SINGLE_FILTER_FLAG = 4
    def __init__(self, tree, dataset, master, parent=None):
        QGroupBox.__init__(self, parent)
        Control.__init__(self, tree, dataset, master)
        
        self.setFlat(True)
#        self.setLayout(QFormLayout())
#        self.enableCB = QCheckBox(getattr(tree, "displayName", ""), self)
        self.attributes = []
        self.filters = []
        
    def setCollection(self, tree, dataset, flags=0):
        self.tree = tree
        self.dataset = dataset
        self.collectionFlags = flags

    
def isFilterCollection(tree):
    """ In case all AttributeDescription nodes contain pointers to filters
    (case of downstream/upstream_flank)
    """
    if tree.tag == "FilterCollection":
        return True
    return all(["pointerFilter" in desc.attributes  for desc in tree.elements("AttributeDescription")])


class AttributeWidget(QCheckBox, Control):
    """ Single attribute selection widget
    """
    def __init__(self, tree, dataset, master, parent=None):
        QCheckBox.__init__(self, parent)
        Control.__init__(self, tree, dataset, master)
        
        self.setText(getattr(tree, "displayName", ""))
        self.setChecked(getattr(tree, "default", "false") == "true")
        
        if hasattr(tree, "description"):
            self.setToolTip(tree.description)
    
        
    def query(self):
        if self.isChecked():
            return [("Attribute", self.tree, self.isChecked())]
        else:
            return []
    
    
    def setControlValue(self, name, value):
        self.setChecked(value == "true")
        
    
class AttributeCollectionWidget(CollectionWidget):
    def __init__(self, tree, dataset, master, parent=None):
        CollectionWidget.__init__(self, tree, dataset, master, parent)
        self.setLayout(QGridLayout(self))
        self.setCollection(tree, dataset)
        
    def setCollection(self, collection, dataset, flags=0):
        CollectionWidget.setCollection(self, collection, dataset, flags)
        self.setTitle(getattr(collection, "displayName", ""))
        
        if hasattr(collection, "description"):
            self.setToolTip(collection.description)
            
        attributes = [attr for attr in collection.elements("AttributeDescription") if not is_hidden(attr)]
        
        numAttrs = max(len(attributes) / 2 + 1, 1)
        i = 0
        for attr in attributes:
            if attr.is_pointer():
                try:
                    dataset, newattr = attr.get_pointed()
                except ValueError, ex:
                    newattr = None
                    
                if not newattr:
                    continue
                attr = newattr
                
            attr_widget = AttributeWidget(attr, self.dataset, self.master, self)
            self.layout().addWidget(attr_widget, i % numAttrs, i / numAttrs)
            self.addSubControl(attr, attr_widget)
            i += 1
            
        
def filterType(filter):
    return getattr(filter, "displayType", ""), getattr(filter, "style", ""), getattr(filter, "multipleValues", "")

 
class FilterCollectionWidget(CollectionWidget):
    def __init__(self, tree, dataset, master, parent=None):
        CollectionWidget.__init__(self, tree, dataset, master, parent)
        self.setLayout(QGridLayout())
#        self.setLayout(QFormLayout())
#        self.layout().setLabelAlignment(Qt.AlignLeft | Qt.AlignVCenter)
#        self.layout().setFormAlignment(Qt.AlignHCenter | Qt.AlignTop)
        
        self.layout().setContentsMargins(5, 5, 5, 5)
        self.layout().setColumnStretch(0, 10)
        self.layout().setColumnStretch(1, 10)
        self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        self.setSizePolicy(QSizePolicy.Preferred, QSizePolicy.Preferred)
        self.setCollection(tree, dataset)
        
    def setCollection(self, collection, dataset, flags=0):
        CollectionWidget.setCollection(self, collection, dataset, flags)
        filters = [f for f in collection.elements("FilterDescription") if not is_hidden(f)] + \
                  [f for f in collection.elements("AttributeDescription") if not is_hidden(f)] # in case of pointers to filters (upstream/downstream flank)
        self.enableCB = QCheckBox(getattr(collection, "displayName", ""), self)
        
        if hasattr(collection, "description"):
            self.setToolTip(collection.description)
            
        if len(filters) == 1 and (not hasattr(filters[0], "displayName") or \
                getattr(filters[0], "displayName", "") == getattr(collection, "displayName", "")):
            flags = flags | self.SINGLE_FILTER_FLAG
            
        i = 0
        if not flags & self.SINGLE_FILTER_FLAG:
            self.layout().addWidget(self.enableCB, 0, 0)
            i += 1
#            self.layout().addRow(self.enableCB, QWidget(self))
            
        for filter in filters:
            fType = getattr(filter, "type", None)
            if filter.is_pointer():
                try:
                    dataset, newfilter  = filter.get_pointed()
                except ValueError, ex:
                    newfilter = None
                    
                if not newfilter:
                    continue
                filter = newfilter
                fType = getattr(filter, "type", None) if fType is None else fType
            label, filter_widget = self.buildFilter(filter, flags, fTypeHint=fType)
            if isinstance(label, basestring):
                label = QLabel(label) 
            
            self.layout().addWidget(label,i, 0)
            self.layout().addWidget(filter_widget, i, 1)
            i += 1
#            self.layout().addRow(label, filter_widget)
            self.addSubControl(filter, filter_widget)
        if self.layout().count() == 0:
            self.layout().addWidget(self.enableCB, 0, 0)
            
            
    def buildFilter(self, filter, flags, fTypeHint=None):
        if flags & self.SINGLE_FILTER_FLAG:
            label = self.enableCB
        else:
            label = getattr(filter, "displayName", "")
            
        fType = filterType(filter)
        if fTypeHint == "drop_down_basic_filter":
            fType = ("list", "menu", "")
        filter_widget = None
        
        if fType == ("text", "", ""):
            filter_widget = TextFieldFilter(filter, self.dataset, self.master, self)
            
        elif fType == ("text", "", "1"):
            filter_widget = IdListFilter(filter, self.dataset, self.master, self)
            
        elif fType == ("list", "radio", ""):
            filter_widget = RadioBooleanFilter(filter, self.dataset, self.master, self)
            
        elif fType == ("list", "menu", ""):
#            if getattr(filter, "graph", "0") == "1":
#                filter_widget = GraphDropDownFilter(filter, self.dataset, self.master, self)
#            else:
#                filter_widget = DropDownFilter(filter, self.dataset, self.master, self)
            filter_widget = DropDownFilter(filter, self.dataset, self.master, self)
            
        elif fType == ("list", "menu", "1"):
            filter_widget = MultiSelectListFilter(filter, self.dataset, self.master, self) 
            
        elif fType == ("container", "", ""):
            fType = set(map(filterType, filter.elements_top("Option")))
            if len(fType) != 1:
                warnings.warn("Multiple filter types in a single container!" + str(fType))
            fType = fType.pop()
#            if fType == ("text", "", ""):
#                filter_widget = DropDownTextFilter(filter, self.dataset, self.master, self)
            if fType[0] == "text": #("text", "", "1"):
                filter_widget = DropDownIdListFilter(filter, self.dataset, self.master, self)
            elif fType == ("list", "radio", ""):
                filter_widget = DropDownRadioBooleanFilter(filter, self.dataset, self.master, self)
        
        if filter_widget is None:
            warnings.warn("Unknown filter type '%s' %s'" % (repr(fType), repr(filter)))
            filter_widget = UnknownFilter(filter, self.dataset, self.master, self)
        
        filter_widget.setMaximumWidth(400)
        return label, filter_widget 
    
    def query(self):
        if self.enableCB.isChecked():
            return CollectionWidget.query(self)
        else:
            return []
            
        
class GroupWidget(QGroupBox, Control):
    def __init__(self, tree, dataset, master, parent=None):
        QGroupBox.__init__(self, parent)
        Control.__init__(self, tree, dataset, master)
        self.setLayout(QVBoxLayout())
        
        if tree:
            self.setGroup(tree)
        
    def setGroup(self, group):
        self.setTitle(getattr(group, "displayName", ""))
        if hasattr(group, "description"):
            self.setToolTip(group.description)
        self.setCheckable(True)
        self.setChecked(True)
        
        if group.tag == "FilterGroup":
            collections = group.elements("FilterCollection")
        else:
            collections = group.elements("AttributeCollection")
        for collection in collections: 
            if isFilterCollection(collection):
                collection_widget = FilterCollectionWidget(collection, self.dataset, self.master, self)
            else:
                collection_widget = AttributeCollectionWidget(collection, self.dataset, self.master, self)
                
            self.layout().addWidget(collection_widget)
            self.addSubControl(collection, collection_widget)
            
            
        
class PageWidget(QFrame, Control):
    def __init__(self, tree, dataset, master, parent=None):
        QFrame.__init__(self, parent)
        Control.__init__(self, tree, dataset, master)
        self.setLayout(QVBoxLayout())
        self.outFormats = getattr(tree, "outFormats", "tsv")
        
        if tree:
            self.setPage(tree)
        
    def setPage(self, tree):
        if tree.tag == "FilterPage":
            groups = tree.elements("FilterGroup")
        else:
            groups = tree.elements("AttributeGroup")
            
        for group in groups:
            group_widget = GroupWidget(group, self.dataset, self.master, self)
            self.layout().addWidget(group_widget)
            self.addSubControl(group, group_widget) 
            
        self.layout().addStretch(10)
        
        if hasattr(tree, "description"):
            self.setToolTip(tree.description)
        

    
class OWBioMart(OWWidget):
    SHOW_FILTERS = True
    
    settingsList = ["selectedDataset"]
    def __init__(self, parent=None, signalManager=None, title="BioMart"):
        OWWidget.__init__(self, parent, signalManager, title)
        
        self.inputs = [("Input ids", ExampleTable, self.setData)]
        self.outputs = [("Example Table", ExampleTable)]
        
        self.selectedDatabase = None
        self.selectedDataset = None
        self.lastSelectedDatasets = (None, None)
        
        self.idAttr = 0
        self.useAttrNames = False
        self.uniqueRows = True
        
        self.loadSettings()
        
        OWGUI.button(OWGUI.widgetBox(self.controlArea, "Cache", addSpace=True),
                     self, "Clear cache",
                     tooltip="Clear saved query results",
                     callback = self.clearCache)
        
        
        self.martsCombo = OWGUI.comboBox(self.controlArea, self, "selectedDatabase", "Database",
                                         callback=self.setSelectedMart,
                                         addSpace=True)
        self.martsCombo.setMaximumWidth(250)
        
        self.datasetsCombo = OWGUI.comboBox(self.controlArea, self, "selectedDataset", "Dataset",
                                            callback=self.setSelectedDataset,
                                            addSpace=True)
        self.datasetsCombo.setMaximumWidth(250)
        
        box = OWGUI.widgetBox(self.controlArea, "Input Ids")
        
        cb = OWGUI.checkBox(box, self, "useAttrNames", "Use attribute names", 
                            tooltip="Use attribute names for ids",
                            callback=self.updateInputIds)
        
        self.idAttrCB = OWGUI.comboBox(box, self, "idAttr", 
                                       tooltip="Use attribute values from ...",
                                       callback=self.updateInputIds)
        
        cb.disables = [(-1, self.idAttrCB)]
        cb.makeConsistent()
        
#        self.idListView = QListView()
#        self.idListView.setSelectionMode(QListView.ExtendedSelection)
#        self.idListView.setModel(QStringListModel(self))
#        box.layout().addWidget(self.idListView)

        self.idText = QPlainTextEdit()
        self.idText.setReadOnly(True)
#        self.idText.setTextInteractionFlags(Qt.TextSelectableByKeyboard)
        box.layout().addWidget(self.idText)
        
        OWGUI.rubber(self.controlArea)
        
        box = OWGUI.widgetBox(self.controlArea, "Results")
        OWGUI.checkBox(box, self, "uniqueRows", "Unique results only",
                       tooltip="Return unique results only.",
                       )
        self.commitButton = OWGUI.button(box, self, "Get Results",
                                         callback=self.commit,
                                         tooltip="Query the BioMart server for results and output the results",
                                         autoDefault=True)
        self.commitButton.setEnabled(False)
        
        self.mainWidget = OWGUI.widgetBox(self.mainArea, orientation=QStackedLayout())
        
        self.mainTab = QTabWidget()
        
        self.mainWidget.layout().addWidget(self.mainTab)
        
        self.attributesConfigurationBox = OWGUI.createTabPage(self.mainTab, "Attributes")#, canScroll=True)
        if self.SHOW_FILTERS:
            self.filtersConfigurationBox = OWGUI.createTabPage(self.mainTab, "Filters")#, canScroll=True)

        self.myThread = OWConcurrent.WorkerThread()
        self.myThread.start()
        
        self.__thread = self.thread() # For assert(QThread.currentThread() is self.thread()) to work
        
        self.connect(self.myThread, SIGNAL("started()"), lambda :sys.stderr.write("Thread started\n"))
        self.connect(self.myThread, SIGNAL("finished()"), lambda :sys.stderr.write("Thread finished\n"))
        
        self.error(0)
        self.setEnabled(False)
        self.get_registry_async = OWConcurrent.createTask(self._get_registry,
                                      onResult=self.setBioMartRegistry,
                                      onFinished=self.onFinished,
                                      thread=self.myThread)
        
        
        self.resize(800, 600)
        
        self.candidateIdAttrs = []
        self._afterInitQueue = []
        self.data = None
        
        try:
            from Bio import SeqIO
            self.hasBiopython = True
        except Exception, ex:
            self.warning(100, "Biopython package not found.\nTo retrieve FASTA sequence data from BioMart install Biopython.")
            self.hasBiopython = False
        
        
    @staticmethod
    def _get_registry(url=None, precache=True):
        con = obiBioMart.BioMartConnection(url)
        reg = obiBioMart.BioMartRegistry(con)
        if precache:
            marts = reg.marts()
        return reg
        
    @pyqtSignature("onFinished(QString)")
    def onFinished(self, status):
        assert(QThread.currentThread() is self.thread())
        if str(status).lower() != "ok":
            print >> sys.stderr, "AsyncCall failed with message:", status
            self.error(0, str(status))
        self.setEnabled(True)
            
    @pyqtSignature("setBioMartRegistry(PyQt_PyObject)")
    def setBioMartRegistry(self, registry):
        assert(QThread.currentThread() is self.thread())
        self.registry = registry
        self.marts = [mart for mart in self.registry.marts() if getattr(mart, "visible", "0") != "0"]
        last, _ = self.lastSelectedDatasets 
        for mart in self.marts:
            self.martsCombo.addItem(mart.displayName)
            
    def setSelectedMart(self):
        self.mart = self.marts[self.selectedDatabase]
        self.error(0)
        self.setEnabled(False)
        self.get_datasets_async = OWConcurrent.createTask(self.mart.datasets,
                                    onResult=self.setBioMartDatasets,
                                    onFinished=self.onFinished,
                                    thread=self.myThread)
        

    @pyqtSignature("setBioMartDatasets(PyQt_PyObject)")
    def setBioMartDatasets(self, datasets):
        assert(QThread.currentThread() is self.thread())
        self.datasets = [data for data in datasets if getattr(data,"visible", "0") != "0"]
        self.datasetsCombo.clear()
        self.datasetsCombo.addItems([data.displayName for data in self.datasets])
            
    def setSelectedDataset(self):
        self.dataset = self.datasets[self.selectedDataset]
        self.error(0)
        
        def get_configuration(dataset):
            """ Only get the response, do not yet parse
            """
            connection = dataset.connection
            stream = connection.configuration(dataset=dataset.internalName, virtualSchema=dataset.virtualSchema)
            response = stream.read()
            return response
        
        self.setEnabled(False)
        
        self.get_configuration_async = OWConcurrent.createTask(get_configuration, (self.dataset,),
                                            onResult=self.setBioMartConfiguration,
                                            onFinished=self.onFinished,
                                            thread=self.myThread)
        
        
    @pyqtSignature("setBioMartConfiguration(PyQt_PyObject)")
    def setBioMartConfiguration(self, configuration):
        assert(QThread.currentThread() is self.thread())
        
        doc = obiBioMart.parseXML(configuration)
        config = list(doc.elements("DatasetConfig"))[0]
        configuration = obiBioMart.DatasetConfig(self.registry, config.tag, config.attributes, config.children)
        
        self.clearConfiguration()
        
        self.configuration = configuration
        
        def hidden(tree):
            return getattr(tree, "hidden", "false") != "false" or getattr(tree, "hideDisplay", "false") != "false"
        
        self.attributePagesTabWidget = tabs =  OWGUI.tabWidget(self.attributesConfigurationBox)
        
        for page in configuration.elements("AttributePage"):
            if not hidden(page):
                page_widget = PageWidget(page, self.dataset, self)
                OWGUI.createTabPage(tabs, getattr(page, "displayName", ""), widgetToAdd=page_widget, canScroll=True)
        
        if self.SHOW_FILTERS:
            self.filterPagesTabWidget = tabs = OWGUI.tabWidget(self.filtersConfigurationBox)
            for page in configuration.elements("FilterPage"):
                if not hidden(page):
                    page_widget = PageWidget(page, self.dataset, self)
                    OWGUI.createTabPage(tabs, getattr(page, "displayName", ""), widgetToAdd=page_widget, canScroll=True)
                    
        self.afterInit()
        
        self.commitButton.setEnabled(True)
    
    def clearConfiguration_(self):
        while True:
            w = self.attributesConfigurationBox.layout().takeAt(0)
            if isinstance(w, QWidgetItem):
                w = w.widget()
            if not w:
                break
            if isinstance(w, QWidget):
                w.setParent(None)
        
        while True:
            w = self.filtersConfigurationBox.layout().takeAt(0)
            if isinstance(w, QWidgetItem):
                w = w.widget()
            if not w:
                break
            if isinstance(w, QWidget):
                w.setParent(None)
            pass
        
    def clearConfiguration(self):
        self.mainTab = QTabWidget()
        self.mainWidget.layout().addWidget(self.mainTab)
        self.mainWidget.layout().setCurrentWidget(self.mainTab)
        
        self.attributesConfigurationBox = OWGUI.createTabPage(self.mainTab, "Attributes")#, canScroll=True)
        if self.SHOW_FILTERS:
            self.filtersConfigurationBox = OWGUI.createTabPage(self.mainTab, "Filters")#, canScroll=True)
            
            
    def count(self):
        filters = []
        count = self.dataset.get_count(filters = filters)
        
        
    def commit(self):
        pageconf = self.attributePagesTabWidget.currentWidget().widget()
        format = pageconf.outFormats
        
        self.error(100)
        if not self.hasBiopython and format.lower() == "fasta":
            self.error(100, "Cannot parse FASTA format")
            return
        
        query = pageconf.query()
        bydatasets = defaultdict(lambda : ([], []))

        version = getattr(self.configuration, "softwareVersion", "0.4")
        for conftype, tree, val in query:
            dataset = self.dataset
#            if version > "0.4":
#                dataset = self.dataset
#            else:
#                widget = self.findChild(QWidget, tree.internalName)
#                if widget:
#                    dataset = widget.dataset
                
            if conftype == "Attribute":
                bydatasets[dataset][0].append(tree.internalName)
            elif conftype == "Filter":
                bydatasets[dataset][1].append((tree.internalName, val))
                
        if self.SHOW_FILTERS:
            pageconf = self.filterPagesTabWidget.currentWidget().widget()
#            configuration =  pageconf.get_configuration()
            query = pageconf.query()
    
            for conftype, tree, val in query:
                dataset = self.dataset
#                if version > "0.4":
#                    dataset = self.dataset
#                else:
#                    widget = self.findChild(QWidget, tree.internalName)
#                    if widget:
#                        dataset = widget.dataset
                    
                if conftype == "Attribute":
                    bydatasets[dataset][0].append(tree.internalName)
                elif conftype == "Filter":
                    bydatasets[dataset][1].append((tree.internalName, val))
                
        query = self.registry.query(format="TSV" if "tsv" in format.lower() else format.upper(), uniqueRows=self.uniqueRows)
        
        for dataset, (attributes, filters) in bydatasets.items():
            query.set_dataset(dataset if dataset else self.dataset)
            for attr in attributes:
                query.add_attribute(attr)
            for filter, value in filters:
                query.add_filter(filter, value)
        
        self.error(0)
        self.setEnabled(False)
        self.run_query_async = OWConcurrent.createTask(query.get_example_table,
                                            onResult=self.dataReady,
                                            onFinished=self.onFinished,
                                            thread=self.myThread)

    
    def dataReady(self, data):
        self.setEnabled(True)
        self.send("Example Table", data)
        
    def pushAction(self, action):
        ref = action.ref
        ref_widget = self.findChild(QWidget, ref)
        if hasattr(ref_widget, "setOptions"):
            ref_widget.setOptions(action.subelements_top("Option"))
            
    def registerDelayedCall(self, call):
        self._afterInitQueue.append(call)
        
    def afterInit(self):
        while self._afterInitQueue:
            call = self._afterInitQueue.pop(0)
            call()
            
    def setData(self, data =None):
        self.data = data
        self.idAttrCB.clear()
        attrs = data.domain.variables + data.domain.getmetas().values()
        attrs = [attr for attr in attrs if isinstance(attr, orange.StringVariable)]
        self.candidateIdAttrs = attrs
        self.idAttrCB.addItems([attr.name for attr in attrs])
        
        self.idAttr = min(self.candidateIdAttrs, len(self.candidateIdAttrs) - 1)
        
        self.updateInputIds()
    
    def updateInputIds(self):
        if self.data:
            if self.useAttrNames:
                names = [attr.name for attr in self.data.domain.attributes]
            elif self.candidateIdAttrs:
                attr = self.candidateIdAttrs[self.idAttr]
                names = [str(ex[attr]) for ex in self.data if not ex[attr].isSpecial()]
            else:
                names = []
        else:
            names = []
            
#        self.idListView.model().setStringList(names)
        self.idText.setPlainText("\n".join(names))
        
        
    def clearCache(self):
        self.registry.connection.clearCache()
    
if __name__ == "__main__":
    app = QApplication(sys.argv)
    ow = OWBioMart()
    ow.show()
    
    data = orange.ExampleTable("../../../doc/datasets/brown-selected")
    ow.setData(data)
    app.exec_()
    ow.saveSettings()



from __future__ import absolute_import

from .arrayexpress import *

import warnings

"""
Gene Expression Atlas
---------------------

.. WARNING:: Deprecated, use ``obiGeneAtlas`` instead.

`Gene Expression Atlas <http://www.ebi.ac.uk/gxa/>`_ is a curated subset of
gene expression experiments in Array Express Archive.

Use `query_atlas_simple` for simple querys.

Example (query human genes for experiments in which they are up regulated) ::

    >>> obiArrayExpress.query_atlas_simple(genes=["SORL1", "PSIP1", "CDKN1C"], regulation="up", organism="Homo sapiens")
    {u'...

Or use the `AtlasCondition` subclasses in this module to construct a more
advanced query and use the `query_atlas` function.

Example (query human genes annotated to the GO term 'transporter activity'
that are up regulated in the liver in at least three experiments) ::

    >>> go_cond = AtlasConditionGeneProperty("Goterm", "Is", "transporter activity")
    >>> liver_cond = AtlasConditionExperimentalFactor("Organism_part", "up", 3, "liver")
    >>> org_cond = AtlasConditionOrganism("Homo sapiens")
    >>> cond_list = AtlasConditionList([go_cond, liver_cond, org_cond])
    >>> query_atlas(cond_list)
    {u'...

"""


def _open_shelve(filename, flag="r"):
    dirname = os.path.dirname(filename)
    if not os.path.isdir(dirname):
        os.makedirs(dirname)
    exists = os.path.exists(filename)
    if flag in ["r", "w"] and not exists:
        # needs to be created first
        # XXX: Race condition
        s = shelve.open(filename, "c")
        s.close()

    return shelve.open(filename, flag)


@contextmanager
def _fake_closing(obj):
    yield obj


class GeneExpressionAtlasConenction(object):

    """
    A connection to Gene Expression Atlas database.

    :param address:
        Address of the GXA server (default: http://www.ebi.ac.uk/gxa/api/deprecated).
    :param timeout:
        Socket timeout (default 30).
    :param cache:
        A dict like object to use as a cache.

    """
    DEFAULT_ADDRESS = "http://www-test.ebi.ac.uk/gxa/api/deprecated"

    DEFAULT_CACHE = serverfiles.localpath(
        "ArrayExpress", "GeneAtlasCache.shelve")

    def __init__(self, address=None, timeout=30, cache=None):
        """
        Initialize the connection.

        """
        self.address = address if address is not None else self.DEFAULT_ADDRESS
        self.timeout = timeout
        if cache is None:
            cache = self.DEFAULT_CACHE

        self.cache = cache

    def query(self, condition, format="json", start=None, rows=None, indent=False):
        url = self.address + "?" + condition.rest()
        if start is not None and rows is not None:
            url += "&start={0}&rows={1}".format(start, rows)
        url += "&format={0}".format(format)
        if indent:
            url += "&indent"
#        print url
        if self.cache is not None:
            return self._query_cached(url)
        else:
            return urllib2.urlopen(url)

    def _query_cached(self, url):
        if self.cache is not None:
            with self.open_cache("r") as cache:
                if url in cache:
                    return StringIO(cache[url])

            response = urllib2.urlopen(url)
            contents = response.read()
            with self.open_cache("w") as cache:
                cache[url] = contents

            return StringIO(contents)
        else:
            return urllib2.urlopen(url)

    def open_cache(self, flag="r"):
        """
        Return a context manager for a dict like object.
        """
        if isinstance(self.cache, basestring):
            try:
                return closing(_open_shelve(self.cache, flag))
            except Exception:
                return _fake_closing({})
        else:
            return _fake_closing(self.cache)


# Names of all Gene Property filter names
GENE_FILTERS = \
    ["Name",  # Gene name
     "Goterm",  # Gene Ontology Term
     "Interproterm",  # InterPro Term
     "Disease",  # Gene-Disease Assocation
     "Keyword",  # Gene Keyword
     "Protein",  # Protein

     "Dbxref",  # Other Database Cross-Refs
     "Embl",  # EMBL-Bank ID
     "Ensfamily",  # Ensembl Family
     "Ensgene",  # Ensembl Gene ID

     "Ensprotein",  # Ensembl Protein ID
     "Enstranscript",  # Ensembl Transcript ID
     "Goid",  # Gene Ontology ID
     "Image",  # IMAGE ID
     "Interproid",  # InterPro ID
     "Locuslink",  # Entrez Gene ID

     "Omimid",  # OMIM ID
     "Orf",  # ORF
     "Refseq",  # RefSeq ID
     "Unigene",  # UniGene ID
     "Uniprot",  # UniProt Accession

     "Hmdb",  # HMDB ID
     "Chebi",  # ChEBI ID
     "Cas",  # CAS
     "Uniprotmetenz",  # Uniprotmetenz
     "Gene",  # Gene Name or Identifier
     "Synonym",  # Gene Synonym
     ]

# Valid Gene Property filter qualifiers
GENE_FILTER_QUALIFIERS =\
    ["Is",
     "IsNot"
     ]

# Organisms in the Atlas
ATLAS_ORGANISMS = \
    ["Anopheles gambiae",
     "Arabidopsis thaliana",
     "Bos taurus",
     "Caenorhabditis elegans",
     "Danio rerio",
     "Drosophila melanogaster",
     "Epstein barr virus",
     "Gallus gallus",
     "Homo sapiens",
     "Human cytomegalovirus",
     "Kaposi sarcoma-associated herpesvirus",
     "Mus musculus",
     "Rattus norvegicus",
     "Saccharomyces cerevisiae",
     "Schizosaccharomyces pombe",
     #     "Unknown",
     "Xenopus laevis"
     ]

#_COMMON_TAXIDS = \
#    {"Anopheles gambiae",
#     "Arabidopsis thaliana",
#     "Bos taurus",
#     "Caenorhabditis elegans",
#     "Danio rerio",
#     "Drosophila melanogaster",
#     "Epstein barr virus",
#     "Gallus gallus",
#     "Homo sapiens",
#     "Human cytomegalovirus",
#     "Kaposi sarcoma-associated herpesvirus",
#     "Mus musculus",
#     "Rattus norvegicus",
#     "Saccharomyces cerevisiae",
#     "Schizosaccharomyces pombe",
# "Unknown",
#     "Xenopus laevis"
#     }


def ef_ontology():
    """ Return the `EF <http://www.ebi.ac.uk/efo/>`_ (Experimental Factor) ontology
    """
    from . import obiOntology
#    return obiOntology.OBOOntology(urllib2.urlopen("http://efo.svn.sourceforge.net/svnroot/efo/trunk/src/efoinobo/efo.obo"))
    # Should this be in the OBOFoundry (Ontology) domain
    try:
        file = open(serverfiles.localpath_download("ArrayExpress", "efo.obo"), "rb")
    except urllib2.HTTPError:
        file = urllib2.urlopen("http://efo.svn.sourceforge.net/svnroot/efo/trunk/src/efoinobo/efo.obo")
    return obiOntology.OBOOntology(file)


class AtlasCondition(object):

    """ Base class for Gene Expression Atlas query condition
    """

    def validate(self):
        """ Validate condition in a subclass.
        """
        raise NotImplementedError

    def rest(self):
        """ Return a REST query part in a subclass.
        """
        raise NotImplementedError


class AtlasConditionList(list, AtlasCondition):

    """ A list of AtlasCondition instances.
    """

    def validate(self):
        for item in self:
            item.validate()

    def rest(self):
        return "&".join(cond.rest() for cond in self)


class AtlasConditionGeneProperty(AtlasCondition):

    """ An atlas gene filter condition.

    :param property: Property of the gene. If None or "" all properties 
        will be searched.
    :param qualifier: Qualifier can be 'Is' or 'IsNot'
    :param value: The value to search for.

    Example ::

        >>> # Condition on a gene name
        >>> condition = AtlasConditionGeneProperty("Name", "Is", "AS3MT")
        >>> # Condition on genes from a GO Term
        >>> condition = AtlasConditionGeneProperty("Goterm", "Is", "p53 binding")
        >>> # Condition on disease association
        >>> condition = AtlasConditionGeneProperty("Disease", "Is", "cancer")

    """

    def __init__(self, property, qualifier, value):
        self.property = property or ""
        self.qualifier = qualifier
        if isinstance(value, basestring):
            self.value = value.replace(" ", "+")
        elif isinstance(value, list):
            self.value = "+".join(value)
        else:
            raise ValueError(value)

        self.validate()

    def validate(self):
        assert(self.property in GENE_FILTERS + [""])
        assert(self.qualifier in GENE_FILTER_QUALIFIERS + [""])

    def rest(self):
        return "gene{property}{qualifier}={value}".format(**self.__dict__)


class AtlasConditionExperimentalFactor(AtlasCondition):

    """ An atlas experimental factor filter condition.

    :param factor: EFO experiamntal factor
    :param regulation: "up", "down", "updown", "any" or "none"
    :param n: Minimum number of of experimants with this condition
    :param value: Experimantal factor value

    Example ::

        >>> # Any genes up regulated in at least 3 experiments involving cancer.
        >>> condition = AtlasConditionExperimentalFactor("", "up", 3, "cancer")
        >>> # Only genes which are up/down regulated in the heart in at least one experiment. 
        >>> condition = AtlasConditionExperimentalFactor("Organism_part", "updown", 1, "heart")

    """

    def __init__(self, factor, regulation, n, value):
        self.factor = factor
        self.regulation = regulation
        self.n = n
        self.value = value
        self.validate()

    def validate(self):
        # TODO: validate the factor and value
#        assert(self.factor in ef_ontology())
        assert(self.regulation in ["up", "down", "updown"])

    def rest(self):
        return "{regulation}{n}In{factor}={value}".format(**self.__dict__)


class AtlasConditionOrganism(AtlasCondition):

    """ Condition on organism.
    """

    def __init__(self, organism):
        self.organism = organism
        self.validate()

    def validate(self):
        assert(self.organism in ATLAS_ORGANISMS)

    def rest(self):
        return "species={0}".format(self.organism.replace(" ", "+").lower())


class AtlasConditionExperiment(AtlasCondition):

    """ Condition on experiement

    :param property: Property of the experiment. If None or "" all properties
        will be searched.
    :param qualifier: Qualifier can be 'Has' or 'HasNot'
    :param value: The value to search for.

    Example ::

        >>> # Condition on a experiemnt acession
        >>> condition = AtlasConditionExperiment("", "", "E-GEOD-24283")
        >>> # Condition on experiments involving lung
        >>> condition = AtlasConditionExperiment("Organism_part", "Has", "lung")

    """
#    EXPERIMENT_FILTERS = [
#                "Organism"
#                "Factor"]

    EXPERIMENT_FILTER_QUALIFIERS = [
        "Has",
        "HasNot"]

    def __init__(self, property, qualifier, value):
        self.property = property
        self.qualifier = qualifier
        if isinstance(value, basestring):
            self.value = value.replace(" ", "+")
        elif isinstance(value, list):
            self.value = "+".join(value)
        else:
            raise ValueError(value)

        self.validate()

    def validate(self):
        # TODO: check to EFO factors
#        assert(self.property in EXPERIMENT_FILTERS + [""])
        assert(self.qualifier in self.EXPERIMENT_FILTER_QUALIFIERS + [""])

    def rest(self):
        return "experiment{property}{qualifier}={value}".format(**self.__dict__)


class GeneAtlasError(ValueError):

    """ An error response from the Atlas server.
    """
    pass


def __check_atlas_error_json(response):
    if "error" in response:
        raise GeneAtlasError(response["error"])
    return response


def __check_atlas_error_xml(response):
    error = response.find("error")
    if error is not None:
        raise GeneAtlasError(error.text)
    return response


def query_atlas_simple(genes=None, regulation=None, organism=None,
                       condition=None, format="json", start=None,
                       rows=None):
    """ A simple Atlas query.

    :param genes: A list of gene names to search for.
    :param regulation: Search for experiments in which `genes` are "up",
        "down", "updown" or "none" regulated. If None all experiments
        are searched.
    :param organism: Search experiments for organism. If None all experiments
        are searched.
    :param condition: An EFO factor value (e.g. "brain")

    Example ::

        >>> query_atlas_simple(genes=['Pou5f1', 'Dppa3'], organism="Mus musculus")
        {u'...

        >>> query_atlas_simple(genes=['Pou5f1', 'Dppa3'], regulation="up", organism="Mus musculus")
        {u'...

        >>> query_atlas_simple(genes=['Pou5f1', 'Dppa3'], regulation="up", condition="liver", organism="Mus musculus")
        {u'...

    """
    warnings.warn("Use 'obiGeneAtlas.run_simple_query' instead.", DeprecationWarning)
    conditions = AtlasConditionList()
    if genes:
        conditions.append(AtlasConditionGeneProperty("Gene", "Is", genes))
    if regulation or condition:
        regulation = "any" if regulation is None else regulation
        condition = "" if condition is None else condition
        conditions.append(AtlasConditionExperimentalFactor("", regulation, 1, condition))
    if organism:
        conditions.append(AtlasConditionOrganism(organism))

    connection = GeneExpressionAtlasConenction()
    results = connection.query(conditions, format=format, start=start,
                               rows=rows)
    if format == "json":
        return parse_json(results)
    else:
        return parse_xml(results)


# TODO: can this be implemented query_atlas(organism="...", Locuslink="...", Chebi="...", up3InCompound="..." downInEFO="...")
# Need a full list of accepted factors


def query_atlas(condition, format="json", start=None, rows=None, indent=False, connection=None):
    """ Query Atlas based on a `condition` (instance of AtlasCondition)

    Example ::

        >>> condition1 = AtlasConditionGeneProperty("Goterm", "Is", "p53 binding")
        >>> condition2 = AtlasConditionExperimentalFactor("Organism_part", "up", 3, "heart")
        >>> condition = AtlasConditionList([condition1, condition2])
        >>> query_atlas(condition)
        {u'...

    """
    warnings.warn("Use 'obiGeneAtlas.run_query' instead.", DeprecationWarning)
    if connection is None:
        connection = GeneExpressionAtlasConenction()
    results = connection.query(condition, format=format, start=start,
                               rows=rows, indent=indent)
    if format == "json":
        response = parse_json(results)
        return __check_atlas_error_json(response)
    else:
        response = parse_xml(results)
        return __check_atlas_error_xml(response)


def get_atlas_summary(genes, organism, connection=None):
    """ Return 3 dictionaries containing a summary of atlas information
    about three experimental factors:

        - Organism Part (OP) 
        - Disease State (DS)
        - Cell type (CT)

    Each dictionary contains query genes as keys. Values are dictionaries
    mapping factor values to a 2-tuple containig the count of up regulated
    and down regulated experiments.

    Example ::

        >>> get_atlas_summary(["RUNX1"], "Homo sapiens")
        ({u'RUNX1': ...

    """
    warnings.warn("Use 'obiGeneAtlas.get_atlas_summary' instead.", DeprecationWarning)
    genes_condition = AtlasConditionGeneProperty("Gene", "Is", genes)
    org_condition = AtlasConditionOrganism(organism)
    condition = AtlasConditionList([genes_condition, org_condition])
    result = query_atlas(condition, format="json", connection=connection)

    org_part = collect_ef_summary(result, "organism_part")
    disease_state = collect_ef_summary(result, "disease_state")
    cell_type = collect_ef_summary(result, "cell_type")

    return dict(org_part), dict(disease_state), dict(cell_type)


def collect_ef_summary(info, ef, summary=None):
    """ Collect the results summary from query_atlas, result for experimental
    factor `ef`.
    """
    if summary is None:
        summary = defaultdict(dict)

    results = info["results"]
    for res in results:
        gene = res["gene"]
        expressions = res["expressions"]
        for expression in expressions:
            if expression["ef"] == ef:
                efv = expression["efv"]
                updown = (expression["upExperiments"],
                          expression["downExperiments"]
                          )

                if any(updown):
                    summary[gene["name"]][efv] = updown

    return summary



from collections import namedtuple

from PyQt4.QtCore import QTimer, QThread, pyqtSlot as Slot

import Orange.data
import Orange.feature
import Orange.network
from Orange.orng.orngDataCaching import data_hints

from Orange.OrangeWidgets import OWWidget
from Orange.OrangeWidgets import OWGUI
from Orange.OrangeWidgets import OWItemModels
from Orange.OrangeWidgets.OWConcurrent import ThreadExecutor, Task, methodinvoke

from .. import ppi, taxonomy, gene

NAME = "Gene Network"
DESCRIPTION = "Extract a gene network for a set of genes."
ICON = "icons/GeneNetwork.svg"

INPUTS = [("Data", Orange.data.Table, "set_data")]
OUTPUTS = [("Network", Orange.network.Graph)]

Source = namedtuple(
    "Source",
    ["name", "constructor", "tax_mapping", "sf_domain", "sf_filename",
     "score_filter"]
)

SOURCES = [
    Source("BioGRID", ppi.BioGRID, ppi.BioGRID.TAXID_MAP,
           "PPI", ppi.BioGRID.SERVER_FILE, False),
    Source("STRING", ppi.STRING, ppi.STRING.TAXID_MAP,
           "PPI", ppi.STRING.FILENAME, True)
]


class OWGeneNetwork(OWWidget.OWWidget):
    settingsList = ["taxid", "use_attr_names", "network_source",
                    "include_neighborhood", "min_score"]
    contextHandlers = {
        "": OWWidget.DomainContextHandler(
            "", ["taxid", "gene_var_index", "use_attr_names"]
        )
    }

    def __init__(self, parent=None, signalManager=None, title="Gene Network"):
        super(OWGeneNetwork, self).__init__(
            parent, signalManager, title, wantMainArea=False,
            resizingEnabled=False
        )

        self.taxid = "9606"
        self.gene_var_index = -1
        self.use_attr_names = False
        self.network_source = 1
        self.include_neighborhood = True
        self.autocommit = False
        self.min_score = 0.9
        self.loadSettings()

        self.taxids = taxonomy.common_taxids()
        self.current_taxid_index = self.taxids.index(self.taxid)

        self.data = None
        self.geneinfo = None
        self.nettask = None
        self._invalidated = False

        box = OWGUI.widgetBox(self.controlArea, "Info")
        self.info = OWGUI.widgetLabel(box, "No data on input\n")

        box = OWGUI.widgetBox(self.controlArea, "Organism")
        self.organism_cb = OWGUI.comboBox(
            box, self, "current_taxid_index",
            items=map(taxonomy.name, self.taxids),
            callback=self._update_organism
        )
        box = OWGUI.widgetBox(self.controlArea, "Genes")
        self.genes_cb = OWGUI.comboBox(
            box, self, "gene_var_index", callback=self._update_query_genes
        )
        self.varmodel = OWItemModels.VariableListModel()
        self.genes_cb.setModel(self.varmodel)

        OWGUI.checkBox(
            box, self, "use_attr_names",
            "Use attribute names",
            callback=self._update_query_genes
        )

        box = OWGUI.widgetBox(self.controlArea, "Network")
        OWGUI.comboBox(
            box, self, "network_source",
            items=[s.name for s in SOURCES],
            callback=self._on_source_db_changed
        )
        OWGUI.checkBox(
            box, self, "include_neighborhood",
            "Include immediate gene neighbors",
            callback=self.invalidate
        )
        self.score_spin = OWGUI.doubleSpin(
            box, self, "min_score", 0.0, 1.0, step=0.001,
            label="Minimal edge score",
            callback=self.invalidate
        )
        self.score_spin.setEnabled(SOURCES[self.network_source].score_filter)

        box = OWGUI.widgetBox(self.controlArea, "Commit")
        OWGUI.button(box, self, "Commit", callback=self.commit, default=True)

        self.executor = ThreadExecutor()

    def set_data(self, data):
        self.closeContext()
        self.data = data
        if data is not None:
            self.varmodel[:] = string_variables(data.domain)
            taxid = data_hints.get_hint(data, "taxid", default=self.taxid)

            if taxid in self.taxids:
                self.set_organism(self.taxids.index(taxid))

            self.use_attr_names = data_hints.get_hint(
                data, "genesinrows", default=self.use_attr_names
            )

            if not (0 <= self.gene_var_index < len(self.varmodel)):
                self.gene_var_index = len(self.varmodel) - 1

            self.openContext("", data)
            self.invalidate()
            self.commit()
        else:
            self.varmodel[:] = []
            self.send("Network", None)

    def set_source_db(self, dbindex):
        self.network_source = dbindex
        self.invalidate()

    def set_organism(self, index):
        self.current_taxid_index = index
        self.taxid = self.taxids[index]
        self.invalidate()

    def set_gene_var(self, index):
        self.gene_var_index = index
        self.invalidate()

    def query_genes(self):
        if self.use_attr_names:
            if self.data is not None:
                return [var.name for var in self.data.domain.attributes]
            else:
                return []
        elif self.gene_var_index >= 0:
            var = self.varmodel[self.gene_var_index]
            genes = [str(inst[var]) for inst in self.data
                     if not inst[var].isSpecial()]
            return list(unique(genes))
        else:
            return []

    def invalidate(self):
        self._invalidated = True

        if self.nettask is not None:
            self.nettask.finished.disconnect(self._on_result_ready)
            self.nettask.future().cancel()
            self.nettask = None

        if self.autocommit:
            QTimer.singleShot(10, self._maybe_commit)

    @Slot()
    def _maybe_commit(self):
        if self._invalidated:
            self.commit()

    @Slot()
    def advance(self):
        self.progressBarValue = (self.progressBarValue + 1) % 100

    @Slot(float)
    def set_progress(self, value):
        self.progressBarValue = value

    def commit(self):
        include_neighborhood = self.include_neighborhood
        query_genes = self.query_genes()
        source = SOURCES[self.network_source]
        if source.score_filter:
            min_score = self.min_score
            assert source.name == "STRING"
            min_score = min_score * 1000
        else:
            min_score = None

        taxid = self.taxid
        progress = methodinvoke(self, "advance")
        if self.geneinfo is None:
            self.geneinfo = self.executor.submit(
                fetch_ncbi_geneinfo, taxid, progress
            )

        geneinfo_f = self.geneinfo
        taxmap = source.tax_mapping
        db_taxid = taxmap.get(taxid, taxid)

        if db_taxid is None:
            raise ValueError("invalid taxid for this network")

        def fetch_network():
            geneinfo = geneinfo_f.result()
            ppidb = fetch_ppidb(source, db_taxid, progress)
            return get_gene_network(ppidb, geneinfo, db_taxid, query_genes,
                                    include_neighborhood=include_neighborhood,
                                    min_score=min_score,
                                    progress=methodinvoke(self, "set_progress", (float,)))

        self.nettask = Task(function=fetch_network)
        self.nettask.finished.connect(self._on_result_ready)

        self.executor.submit(self.nettask)

        self.setBlocking(True)
        self.setEnabled(False)
        self.progressBarInit()
        self._invalidated = False
        self._update_info()

    @Slot(object)
    def _on_result_ready(self,):
        self.progressBarFinished()
        self.setBlocking(False)
        self.setEnabled(True)
        net = self.nettask.result()
        self._update_info()
        self.send("Network", net)

    def _on_source_db_changed(self):
        source = SOURCES[self.network_source]
        self.score_spin.setEnabled(source.score_filter)
        self.invalidate()

    def _update_organism(self):
        self.taxid = self.taxids[self.current_taxid_index]
        if self.geneinfo is not None:
            self.geneinfo.cancel()
        self.geneinfo = None
        self.invalidate()

    def _update_query_genes(self):
        self.invalidate()

    def _update_info(self):
        if self.data is None:
            self.info.setText("No data on input\n")
        else:
            names = self.query_genes()
            lines = ["%i unique genes on input" % len(set(names))]
            if self.nettask is not None:
                if not self.nettask.future().done():
                    lines.append("Retrieving ...")
                else:
                    net = self.nettask.result()
                    lines.append("%i nodes %i edges" %
                                 (len(net.nodes()), len(net.edges())))
            else:
                lines.append("")

            self.info.setText("\n".join(lines))


def unique(seq):
    seen = set()
    for el in seq:
        if el not in seen:
            seen.add(el)
            yield el


def string_variables(domain):
    variables = domain.variables + domain.getmetas().values()
    return [v for v in variables if isinstance(v, Orange.feature.String)]


def multimap_inverse(multimap):
    """
    Return a multimap inverse relation.
    """
    d = {}
    for key, values in multimap.iteritems():
        for v in values:
            d.setdefault(v, []).append(key)
    return d


def ppidb_synonym_mapping(ppidb, taxid):
    keys = ppidb.ids(taxid)
    mapping = {key: ppidb.synonyms(key) for key in keys}
    return multimap_inverse(mapping)


def taxonomy_match(query_taxids, target_taxids):
    taxid_mapping = {}
    target_taxids = set(target_taxids)

    for taxid in query_taxids:
        mapped = taxid_map(taxid, target_taxids)
        taxid_mapping[taxid] = mapped

    return taxid_mapping


def taxid_map(query, targets):
    if query in targets:
        return query

    lineage = taxonomy.lineage(query)
    if any(tid in targets for tid in lineage):
        return set(lineage).intersection(targets).pop()
    else:
        return None

from Orange.utils import serverfiles as sf


def fetch_ppidb(ppisource, taxid, progress=None):
    fname = ppisource.sf_filename

    if "{taxid}" in fname:
        if taxid in ppisource.tax_mapping:
            taxid_m = ppisource.tax_mapping[taxid]
            if taxid_m is None:
                raise ValueError(taxid)
            taxid = taxid_m
        fname = fname.format(taxid=taxid)
        constructor = lambda: ppisource.constructor(taxid)
    else:
        constructor = ppisource.constructor

    sf.localpath_download(
        ppisource.sf_domain, fname, callback=progress, verbose=True
    )
    return constructor()


def fetch_ncbi_geneinfo(taxid, progress=None):
    taxid = gene.NCBIGeneInfo.TAX_MAP.get(taxid, taxid)
    sf.localpath_download(
        "NCBI_geneinfo", "gene_info.{taxid}.db".format(taxid=taxid),
        callback=progress, verbose=True,
    )
    return gene.NCBIGeneInfo(taxid)


def get_gene_network(ppidb, geneinfo, taxid, query_genes,
                     include_neighborhood=True, min_score=None,
                     progress=None):
    if progress is not None:
        progress(1.0)

    # Normalize the names to ppidb primary keys
    matcher = geneinfo.matcher
    query_genes = zip(query_genes, map(matcher.umatch, query_genes))
    synonyms = ppidb_synonym_mapping(ppidb, taxid)

    query_genes = [(query_gene, geneid,
                    synonyms.get(query_gene, synonyms.get(geneid)))
                    for query_gene, geneid in query_genes]

    query = [(syn[0], query_gene)
             for query_gene, _, syn in query_genes if syn]

    net = extract_network(ppidb, dict(query), geneinfo, include_neighborhood,
                          min_score, progress=progress)
    return net


def extract_network(ppidb, query, geneinfo, include_neighborhood=True,
                    min_score=None, progress=None):
    """
    include neighborhood
    """
    from functools import partial
    from collections import defaultdict
    from itertools import count

    if not isinstance(query, dict):
        query = {name: name for name in query}

    report_weights = True
    if isinstance(ppidb, ppi.BioGRID):
        # BioGRID scores are not comparable (they can be p values,
        # confidence scores, ..., i.e. whatever was reported in the source
        # publication)
        report_weights = False
        if min_score is not None:
            raise ValueError("min_score used with BioGrid")

    graph = Orange.network.Graph()
    # node ids in Orange.network.Graph need to be in [0 .. n-1]
    nodeids = defaultdict(partial(next, count()))

    def gi_info(names):
        mapping = [(name, geneinfo.matcher.umatch(name)) for name in names]
        mapping = [(name, match) for name, match in mapping if match]
        entries = [(name, geneinfo[match]) for name, match in mapping]

        if len(entries) > 1:
            # try to resolve conflicts by prioritizing entries whose
            # symbol/gene_id/locus_tag exactly matches the synonym name.
            entries_ = [(name, entry) for name, entry in entries
                        if name in [entry.gene_id, entry.symbol, entry.locus_tag]]
            if len(entries_) == 1:
                entries = entries_

        if len(entries) == 0:
            return None
        elif len(entries) >= 1:
            # Need to report multiple mappings
            return entries[0][1]

    # Add query nodes.
    for key, query_name in query.items():
        nodeid = nodeids[key]
        synonyms = ppidb.synonyms(key)
        entry = gi_info(synonyms)
        graph.add_node(
            nodeid,
            key=key,
            synonyms=synonyms,
            query_name=query_name,
            symbol=entry.symbol if entry is not None else ""
        )

    if include_neighborhood:
        # extend the set of nodes in the network with immediate neighborers
        edges_iter = (edge for key in query for edge in ppidb.edges(key))
        for id1, id2, score in edges_iter:
            if min_score is None or score >= min_score:
                nodeid1 = nodeids[id1]
                nodeid2 = nodeids[id2]
                if nodeid1 not in graph:
                    synonyms1 = ppidb.synonyms(id1)
                    entry1 = gi_info(synonyms1)
                    symbol1 = entry1.symbol if entry1 is not None else ""
                    graph.add_node(
                        nodeid1, key=id1, synonyms=synonyms1,
                        symbol=symbol1
                    )

                if nodeid2 not in graph:
                    synonyms2 = ppidb.synonyms(id2)
                    entry2 = gi_info(synonyms2)
                    symbol2 = entry2.symbol if entry2 is not None else ""
                    graph.add_node(
                        nodeid2, key=id2, synonyms=synonyms2,
                        symbol=symbol2
                    )

    # add edges between nodes
    for i, id1 in enumerate(nodeids.keys()):
        if progress is not None:
            progress(100.0 * i / len(nodeids))

        for _, id2, score in ppidb.edges(id1):
            if id2 in nodeids and (min_score is None or score >= min_score):
                nodeid1 = nodeids[id1]
                nodeid2 = nodeids[id2]
                assert nodeid1 in graph and nodeid2 in graph
                if score is not None and report_weights:
                    graph.add_edge(nodeid1, nodeid2, weight=score)
                else:
                    graph.add_edge(nodeid1, nodeid2)

    nodedomain = Orange.data.Domain(
        [Orange.feature.String("Query name"),  # if applicable
         Orange.feature.String("id"),          # ppidb primary key
         Orange.feature.String("Synonyms"),    # ppidb synonyms
         Orange.feature.String("Symbol"),      # ncbi gene name ??
         Orange.feature.Discrete("source", values=["false", "true"])],
        None
    )

    node_items = sorted(graph.node.items(), key=lambda t: nodeids[t[0]])

    nodeitems = Orange.data.Table(
        nodedomain,
        [[str(node.get("query_name", "")),
          str(node.get("key", "")),
          str(", ".join(node.get("synonyms", []))),
          str(node.get("symbol", nodeid)),
          "true" if "query_name" in node else "false"]
         for nodeid, node in node_items]
    )
    graph.set_items(nodeitems)
    return graph


def main():
    from PyQt4.QtGui import QApplication
    app = QApplication([])
    w = OWGeneNetwork()
    brown = Orange.data.Table("brown-selected")
    w.set_data(Orange.data.Table(brown[:5]))
    w.show()
    app.exec_()
    w.saveSettings()
    return 0

if __name__ == "__main__":
    main()



import distutils
import distutils.sysconfig
import sys
import os

if len(sys.argv) <= 1 or len(sys.argv) > 1 and "remove" not in sys.argv[1]:
    try:
        print "Downloading essential genomics databases"
        if sys.platform == "win32":
            os.system("start " + sys.prefix+"""\\python.exe -c "from Orange.orng import orngServerFiles; orngServerFiles.update_by_tags(tags=['essential'])" """)
        else:
            from Orange.orng import orngServerFiles
            orngServerFiles.update_by_tags(tags=["essential"])
    except Exception, ex:
        print ex
        pass

    print "To download and/or update the databases needed by Orange"
    print "Bioinformatics please use the \"Update Genomics Databases\""
    print "widget in Orange Canvas or use the orngServerFiles module:"
    print
    print "    from Orange.orng import orngServerFiles"
    print "    orgnServerFiles.consoleupdate()"
    print



import Orange.bio.kegg
import Orange.bio.gene

keggorg = Orange.bio.kegg.KEGGOrganism("mmu")
kegg_genes = keggorg.get_genes() 

query = [ "Fndc4", "Itgb8", "Cdc34", "Olfr1403" ] 

gm = Orange.bio.gene.GMKEGG("mmu") #use KEGG aliases for gene matching
gm.set_targets(kegg_genes) #set KEGG gene aliases as targets

for name in query:
    match = gm.umatch(name)
    if match:
    	pwys = keggorg.get_pathways_by_genes([match])
        print name, "is in"
        pathways = [ Orange.bio.kegg.KEGGPathway(p).title for p in pwys ]
        if pathways:
            for a in pathways:
                print ' ', a
        else:
            print '  /'



"""
Check all data files from GEO, find those which include at least N
samples in all sample subsets of at least one sample type. Useful
when, for instance, filtering out the data sets that could be used for
supervised machine learning.
"""

import Orange.bio.geo

def valid(info, n=40):
    """Return a set of subset types containing more than n samples in every subset"""
    invalid = set()
    subsets = set([sinfo["type"] for sinfo in info["subsets"]])
    for sampleinfo in info["subsets"]:
        if len(sampleinfo["sample_id"]) < n:
            invalid.add(sampleinfo["type"])
    return subsets.difference(invalid)

def report(stypes, info):
    """Pretty-print GDS and valid susbset types"""
    for id, sts in stypes:
        print id
        for st in sts:
            print "  %s:" % st,
            gds = info[id]
            print ", ".join(["%s/%d" % (sinfo["description"], len(sinfo["sample_id"])) \
                             for sinfo in gds["subsets"] if sinfo["type"]==st])

gdsinfo = Orange.bio.geo.GDSInfo()
valid_subset_types = [(id, valid(info)) for id, info in gdsinfo.items() if valid(info)]
report(valid_subset_types, gdsinfo)

print 'N =', len(valid_subset_types)



"""
<name>Display Motifs</name>
<description>None.</description>
<contact>Tomaz Curk</contact>
<icon>icons\GenomeMap.png</icon>
<prototype>1</prototype>
"""

from collections import defaultdict

import orange
from Orange.OrangeWidgets import OWGUI
from Orange.OrangeWidgets.OWGraph import *
from Orange.OrangeWidgets.OWTools import *
from Orange.OrangeWidgets.OWWidget import *

class subBarQwtPlotCurve(QwtPlotCurve):
    def __init__(self, text = None):
        QwtPlotCurve.__init__(self, text or "")
        self.color = Qt.black

    def draw(self, p, xMap, yMap, f, t=-1):
        p.setBackgroundMode(Qt.OpaqueMode)
        p.setBrush(self.color)
        p.setPen(self.color)
        
        if t < 0:
            t = self.dataSize() - 1
            f = 0
        if divmod(f, 2)[1] != 0:
            f -= 1
        if divmod(t, 2)[1] == 0:
            t += 1
            
        for i in range(f, t+1, 2):
            px1 = xMap.transform(self.x(i))
            py1 = yMap.transform(self.y(i))
            px2 = xMap.transform(self.x(i+1))
            py2 = yMap.transform(self.y(i+1))
            p.drawRect(px1, py1, (px2 - px1), (py2 - py1))


class OWDisplayMotifs(OWWidget):
    settingsList = []
    def __init__(self,parent=None, signalManager = None):
        OWWidget.__init__(self, parent, signalManager, "&Display Motifs", 0)

        # set default settings
        self.colorBy = None
        self.pvalThresholdIndex = None
        self.pvalThreshold = None

        #load settings
        self.loadSettings()

        # GUI
        self.graph = OWGraph(self.mainArea)
        self.graph.setYRlabels(None)
        self.graph.enableGridXB(0)
        self.graph.enableGridYL(1)
        self.graph.setAxisMaxMinor(QwtPlot.xBottom, 10)
        self.graph.setAxisMaxMajor(QwtPlot.xBottom, 10)
        self.graph.setAxisAutoScale(QwtPlot.xBottom)
        self.graph.setAxisScale(QwtPlot.xBottom, -1020, 0, 0)
        self.mainArea.layout().addWidget(self.graph)
        
        # inputs
        # data and graph temp variables
        self.inputs = [("Examples", ExampleTable, self.cdata, Default), ("Genes", list, self.newGeneList, Default), ("Motifs", list, self.newMotifList, Default)]

        self.data = None
        self.motifLines = []
        self.visibleValues = []
        self.valueToCurve = {}
        self.allGenes = [] ## genes displayed always in same order
        self.geneList = [] ## selected genes
        self.motifList = [] ## selected motifs
        self.valuesPresentInData = []

        self.clusterPostProbThreshold = 0

        # GUI
        self.selValues = OWGUI.widgetBox(self.controlArea, "Values")
        self.selcolorBy = OWGUI.widgetBox(self.controlArea, "Color By")

        self.colorByCombo = OWGUI.comboBox(self.selcolorBy, self, "colorBy",
                                           items=[],
                                           callback=self.colorByChanged)
        
        self.pvalThresholdCombo = OWGUI.comboBox(self.selValues, self, "pvalThresholdIndex",
                                                 items=[],
                                                 callback=self.pvalThresholdChanged)
        
        self.valuesQLB = QListWidget(self.selValues)
        self.valuesQLB.setSelectionMode(QListWidget.MultiSelection)
        self.connect(self.valuesQLB, SIGNAL("itemSelectionChanged()"), self.valuesSelectionChange)
        self.selValues.layout().addWidget(self.valuesQLB)
        
        self.unselectAllQLB = OWGUI.button(self.selValues, self, "Unselect all",
                                           callback = self.unselAll)

    def unselAll(self):
        self.valuesQLB.clearSelection()

    def pvalThresholdChanged(self):
        print self.pvalThresholdIndex
        if self.pvalThresholdIndex:
            self.pvalThreshold = float(self.pvalThresholdCombo.text(self.pvalThresholdIndex))
        else:
            self.pvalThreshold = None
        self.calcMotifsGraph()
        self.updateMotifsGraph()

    def valuesSelectionChange(self):
        visibleOutcomes = []
        for i in range(self.valuesQLB.count()):
            item = self.valuesQLB.item(i)
            if item.isSelected():
                visibleOutcomes.append(str(item.text()))
        self.visibleValues = visibleOutcomes
        self.updateMotifsGraph()

    def updateSelectionChange(self):
        ## make new colors only for those colorBy values present in data
        colors = ColorPaletteHSV(len(self.valuesPresentInData))
        for (i, v) in enumerate(self.valuesPresentInData):
            curve = self.valueToCurve[v]
            curve.color = colors[i]

        self.setValuesNames(self.valuesPresentInData) 
        for i in range(self.valuesQLB.count()):
            item = self.valuesQLB.item(i)
            item.setSelected(str(item.text()) in self.visibleValues)
#            else:
#                self.valuesQLB.setSelected(i, 0)

    def colorByChanged(self):
#        self.graph.removeCurves()
        self.graph.removeDrawingCurves()
        self.valueToCurve = {}

        if self.colorBy == None:
            self.setValuesNames(None)
            return
            
        if self.potentialColorVariables:
            var = self.potentialColorVariables[self.colorBy]
            values = sorted(var.values)
#        values.sort()

        ## generate colors for each colorby value (each color on its own curve)
        colors = ColorPaletteHSV(len(values))
        for (i, v) in enumerate(values):
            ## create curve in graph
            curve = subBarQwtPlotCurve()
            curve.color = colors[i]
            curve.attach(self.graph)
#            ckey = self.graph.insertCurve(curve)
            curve.setStyle(QwtPlotCurve.UserCurve)
            self.valueToCurve[v] = curve

        self.setValuesNames(values)
        self.calcMotifsGraph()

    def setValuesNames(self, values):
        self.valuesQLB.clear()
        if values == None:
            return
        for v in values:
            self.valuesQLB.addItem(QListWidgetItem(QIcon(ColorPixmap(self.valueToCurve[v].color)), v))
            self.valuesQLB.item(self.valuesQLB.count() - 1).setSelected(True)
        

    ## signal processing
    def newGeneList(self, list):
        self.geneList = []
        
        if list is not None:
            self.geneList = list
            self.calcMotifsGraph()
            self.updateMotifsGraph()
 
    def newMotifList(self, list):
        self.motifList = []
        
        if list is not None:
            self.motifList = list
            self.calcMotifsGraph()
            self.updateMotifsGraph()
        
    def cdata(self, data):
        self.data = data
        self.motifLines = []
        self.colorByCombo.clear()
    
        if self.data == None:
            self.colorBy = None
            self.potentialColorVariables = []
            self.colorByChanged()
            self.graph.setYLlabels(None)
        else:
            self.potentialColorVariables = [v for v in self.data.domain.variables + self.data.domain.getmetas().values() \
                                            if v.varType == orange.VarTypes.Discrete]
            for var in self.potentialColorVariables:
                self.colorByCombo.addItem(str(var.name))
    
            ## break motif info according to gene (sequenceID)
            self.motifLines = defaultdict(list)
            for e in self.data:
                geneID = str(e['sequenceID'])
                self.motifLines[geneID].append(e)
                
            self.allGenes = sorted(self.motifLines.keys())
            if self.potentialColorVariables:
                self.colorBy = min(len(self.potentialColorVariables) - 1, self.colorBy or 0)
            else:
                self.colorBy = None
                
            self.colorByChanged()
    
        if len(self.potentialColorVariables) > 0:
            self.colorByCombo.setCurrentIndex(0)

    ## update graph
    def calcMotifsGraph(self):
        graphData = {}
        for (colorKey, curve) in self.valueToCurve.iteritems():
            graphData[colorKey] = [[], []]

        lineCn = 0
        yVals = []
        self.colorByValuesPresentInData = []
        self.valuesPresentInData = []
        for lineKey in self.geneList:
##            if (self.geneList <> None) and (self.geneList <> []) and (lineKey not in self.geneList): continue
            yVals.append( lineKey)
            for e in self.motifLines[lineKey]:
                motifName = str(e['motifName'])#.value
                if self.motifList is not None and self.motifList != [] and motifName not in self.motifList:
                    continue

                motifDistToEnd = -int(e['distToEnd']) #.value)
                postProb = int(round(float(e['pvalue']) * 100))
                dir = str(e['direction'])

                if 1 or (postProb >= self.clusterPostProbThreshold): ## and (chiSquare >= self.motifChiSquareThreshold):
                    colorAttr = self.potentialColorVariables[self.colorBy]
                    colorKey = str(e[colorAttr])
                    if colorKey not in self.valuesPresentInData:
                        self.valuesPresentInData.append(colorKey)
                    # print motifNumber, colorKey
                    # set the point x values
                    graphData[colorKey][0].append(motifDistToEnd)
                    graphData[colorKey][0].append(motifDistToEnd + 5.0)
                    # set the point y values
                    if dir == "1":
                        graphData[colorKey][1].append(lineCn + 0.0)
                        graphData[colorKey][1].append(lineCn + 0.30)
                    elif dir == "-1":
                        graphData[colorKey][1].append(lineCn - 0.30)
                        graphData[colorKey][1].append(lineCn + 0.0)
                    else:
                        graphData[colorKey][1].append(lineCn - 0.30)
                        graphData[colorKey][1].append(lineCn + 0.30)
            lineCn += 1

        vals = reduce(list.__add__, [val[0] for val in graphData.itervalues()], [])
        minX = min(vals or [0]) - 5.0
        maxX = max(vals or [0]) + 5.0
        
        self.graph.setYLlabels(yVals)
        self.graph.setAxisScale(QwtPlot.yLeft, -0.5, len(yVals) - 0.5, 1)
        self.graph.setAxisScale(QwtPlot.xBottom, minX, maxX)

        for (colorKey, curve) in self.valueToCurve.iteritems():
            curve.setData(graphData[colorKey][0], graphData[colorKey][1])
            curve.setVisible(colorKey in self.visibleValues)
        self.graph.replot()
        self.valuesPresentInData.sort()
        self.visibleValues = [v for v in self.valuesPresentInData]
        self.updateSelectionChange()
        
    def updateMotifsGraph(self):
        for (colorKey, curve) in self.valueToCurve.items():
            curve.setVisible(colorKey in self.visibleValues)
        self.graph.replot()

if __name__ == "__main__":
    a = QApplication(sys.argv)
    owdm = OWDisplayMotifs()
    owdm.cdata(orange.ExampleTable(os.path.expanduser("~/Desktop/motif.tab")))
    owdm.newGeneList(["1", "2"])
    owdm.show()
    a.exec_()
    owdm.saveSettings()



"""
<name>Array Express</name>
<description>Array Express datasets<description>

"""

import sys
import os
from datetime import date

import Orange

from OWWidget import *
import OWGUI

from Orange.bio import obiArrayExpress


class OWArrayExpress(OWWidget):
    settingsList = ["current_experiement", "search_string"]

    HEADER_LABELS = ["ID", "Title", "Species", "Assays", "Date"]

    def __init__(self, parent=None, signalManager=None, title="Array Express"):
        OWWidget.__init__(self, parent, signalManager, title)

        self.outputs = [("Data Table", Orange.data.Table)]
        self.current_experiment = None
        self.search_string = ""

        self.loadSettings()

        #####
        # GUI
        #####

        box = OWGUI.widgetBox(self.controlArea, "Info")
        self.info = OWGUI.widgetLabel(box, "\n")

        OWGUI.rubber(self.controlArea)
        OWGUI.button(self.controlArea, self, "Commit", callback=self.commit)

        self.experiments_view = QTreeView(self)
        self.experiments_view.setSortingEnabled(True)
        self.experiments_view.viewport().setMouseTracking(True)
        self.experiments_view.setItemDelegateForColumn(
            0, OWGUI.LinkStyledItemDelegate(self.experiments_view)
        )

        model = QStandardItemModel()
        model.setHorizontalHeaderLabels(self.HEADER_LABELS)

        self.experiments_view.setModel(model)

        self.mainArea.layout().addWidget(self.experiments_view)

        self.setEnabled(False)
        QTimer.singleShot(5, self.fill_experiments)

    def fill_experiments(self):
        self.connection = obiArrayExpress.ArrayExpressConnection()
        self.all_experiments = []
        res = obiArrayExpress.query_experiments(gxa=True)  # only gxa for now
        res = res["experiments"]
        experiments = res["experiment"]
        model = QStandardItemModel(self)
        model.setHorizontalHeaderLabels(self.HEADER_LABELS)

        for exp in experiments:
            accession = str(exp["accession"])
            species = str(exp.get("species", ""))
            title = exp.get("name", "")
            assays = str(exp["assays"])
            date = exp.get("releasedate", "") or ""

            row = map(PyStandardItem,
                      [accession, title, species, assays, date])
            url = "http://www.ebi.ac.uk/arrayexpress/experiments/" + accession
            row[0].setData(QVariant(url), OWGUI.LinkRole)
            if not exp.get("processeddatafiles", {}).get("available", False):
                continue

            model.appendRow(row)

        self.experiments_view.setModel(model)

        self.info.setText("%i experiments" % model.rowCount())
        self.setEnabled(True)

    def commit(self):
        selected = self.experiments_view.selectionModel().selectedRows()

        if selected:
            i = selected[0].row()
            model = self.experiments_view.model()
            item = model.item(i)
            accession = str(item.data(Qt.DisplayRole).toPyObject())
            experiment = obiArrayExpress.ArrayExpressExperiment(accession)
            table = experiment.fgem_to_table()

            self.send("Data Table", table)


class PyStandardItem(QStandardItem):
    def __lt__(self, other):
        return self.data(Qt.DisplayRole).toPyObject() < \
                other.data(Qt.DisplayRole).toPyObject()


if __name__ == "__main__":
    import sys
    app = QApplication(sys.argv)
    w = OWArrayExpress()
    w.show()
    app.exec_()



import Orange
import glob
import re

filenames = glob.glob(Orange.utils.serverfiles.localpath("GEO") + "/GDS*.soft.gz")
m = re.compile("(GDS[0-9]*).soft")
print "%d data files cached:" % len(filenames)
print " ".join([m.search(fn).group(1) for fn in filenames])




from __future__ import absolute_import

import os
import urllib2
import shutil
import gzip
import zipfile
import xml.dom.minidom as minidom
import csv
import sqlite3
import errno
import posixpath
import textwrap

from StringIO import StringIO
from collections import defaultdict, namedtuple
from operator import itemgetter

from Orange.utils import serverfiles
from Orange.utils import ConsoleProgressBar, wget

from . import taxonomy

from .taxonomy import pickled_cache


def mkdir_p(path, mode=0o777):
    try:
        os.makedirs(path, mode)
    except OSError as err:
        if err.errno == errno.EEXIST:
            pass
        else:
            raise


class PPIDatabase(object):
    """
    A general interface for protein-protein interaction database access.

    An example::

        >>> ppidb = MySuperPPIDatabase()
        >>> ppidb.organisms() # List all organisms (taxids)
        ['...

        >>> ppidb.ids() # List all protein ids
        ['...

        >>> ppidb.ids(taxid="9606") # List all human protein ids.
        ['...

        >>> ppidb.links() # List all links
        [('...

    """
    def __init__(self):
        pass

    def organisms(self):
        """
        Return all organism ncbi taxonomy ids contained in this database.
        """
        raise NotImplementedError

    def ids(self, taxid=None):
        """
        Return a list of all protein ids. If `taxid` (as returned by
        `organisms()`) is not ``None`` limit the results to ids to
        this organism only.

        """
        raise NotImplementedError

    def synonyms(self, id):
        """
        Return a list of synonyms for primary `id` (as returned by `ids`).
        """
        raise NotImplementedError

    def all_edges(self, taxid=None):
        """
        Return a list of all edges. If `taxid` is not ``None`` return the
        edges for this organism only.

        """
        raise NotImplementedError

    def edges(self, id1, id2=None):
        """
        Return a list of all edges (a list of 3-tuples (id1, id2, score)).
        """
        raise NotImplementedError

    def all_edges_annotated(self, taxid=None):
        """
        Return a list of all edges annotated. If taxid is not None
        return the edges for this organism only.

        """
        res = []
        for id in self.ids(taxid):
            res.extend(self.edges_annotated(id))
        return res

    def edges_annotated(self, id=None):
        """
        Return a list of all edges annotated.
        """
        raise NotImplementedError

    def search_id(self, name, taxid=None):
        """
        Search the database for protein name. Return a list of matching
        primary ids. Use `taxid` to limit the results to a single organism.

        """
        raise NotImplementedError

    def extract_network(self, ids):
        """
        """
        from Orange import network

        graph = network.Graph()
        for id in ids:
            graph.add_node(id, synonyms=",".join(self.synonyms(id)))

        for id in ids:
            edges = self.edges(id)

            for id1, id2, score in edges:
                graph.add_edge(id1, id2, weight=score)

        return graph

    @classmethod
    def download_data(self):
        """
        Download the latest PPI data for local work.
        """
        raise NotImplementedError


class BioGRID(PPIDatabase):
    """
    Access `BioGRID <http://thebiogrid.org>`_ PPI data.

    Example ::

        >>> biogrid = BioGRID()
        >>> print biogrid.organism() # Print a list of all organism ncbi taxis in BioGRID
        [u'10090',...

        >>> print biogrid.ids(taxid="9606") # Print a set of all human protein ids
        [u'110004'

        >>> print biogrid.synonyms("110004") # Print a list of all synonyms for protein id '110004' as reported by BioGRID
        [u'3803', u'CU464060.2', u'CD158b', u'p58.2', u'CD158B1', u'NKAT6']

        >>>

    """

    SCHEMA = [
        ("links",
         """\
         biogrid_interaction_id text,
         biogrid_id_interactor_a text,
         biogrid_id_interactor_b text,
         experimental_system text,
         experimental_system_type text,
         author text,
         pubmed_id text,
         throughput text,
         score real,
         modification text,
         phenotypes text,
         qualifications text,
         tags text,
         source_database text
         """),
        ("proteins",
         """\
         biogrid_id_interactor text,
         entrez_gene_interactor text,
         systematic_name_interactor text,
         official_symbol_interactor text,
         synonyms_interactor text,
         organism_interactor text,
         """)
    ]

    VERSION = "2.0"

    # All column names in the BioGRID tab2 source table.
    FIELDS = ['biogrid_interaction_id',
              'entrez_gene_interactor_a',
              'entrez_gene_interactor_b',
              'biogrid_id_interactor_a',
              'biogrid_id_interactor_b',
              'systematic_name_interactor_a',
              'systematic_name_interactor_b',
              'official_symbol_interactor_a',
              'official_symbol_interactor_b',
              'synonyms_interactor_a',
              'synonyms_interactor_b',
              'experimental_system',
              'experimental_system_type',
              'author',
              'pubmed_id',
              'organism_interactor_a',
              'organism_interactor_b',
              'throughput',
              'score',
              'modification',
              'phenotypes',
              'qualifications',
              'tags',
              'source_database'
              ]

    DOMAIN = "PPI"
    SERVER_FILE = "BIOGRID-ALL.sqlite"

    TAXID_MAP = {
        "4530": "39947",
        "4932": "559292",
        "562": "511145",
        "5833": "36329",
        "2104": None,
        "4754": None,
        "31033": None
    }

    def __init__(self):
        self.filename = serverfiles.localpath_download(
            self.DOMAIN, self.SERVER_FILE)

        # assert version matches
        self.db = sqlite3.connect(self.filename)
        self.init_db_index()

    def organisms(self):
        cur = self.db.execute("select distinct organism_interactor \n"
                              "from proteins")
        return map(itemgetter(0), cur.fetchall())

    def ids(self, taxid=None):
        """
        Return a list of all protein ids (biogrid_id_interactors).
        If `taxid` is not None limit the results to ids from this organism
        only.

        """
        if taxid is None:
            cur = self.db.execute("""\
                select biogrid_id_interactor
                from proteins""")
        else:
            cur = self.db.execute("""\
                select biogrid_id_interactor
                from proteins
                where organism_interactor=?""",
                (taxid,))

        return [t[0] for t in cur.fetchall()]

    def synonyms(self, id):
        """
        Return a list of synonyms for primary `id`.
        """
        cur = self.db.execute("""\
            select entrez_gene_interactor,
                   systematic_name_interactor,
                   official_symbol_interactor,
                   synonyms_interactor
            from proteins
            where biogrid_id_interactor=?""",
            (id,))
        rec = cur.fetchone()
        if rec:
            synonyms = list(rec[:-1]) + \
                       (rec[-1].split("|") if rec[-1] is not None else [])
            return [s for s in synonyms if s is not None]
        else:
            return []

    def all_edges(self, taxid=None):
        """
        Return a list of all edges. If taxid is not None return the
        edges for this organism only.

        """
        if taxid is not None:
            cur = self.db.execute("""\
                select biogrid_id_interactor_a, biogrid_id_interactor_a, score
                from links left join proteins on
                    biogrid_id_interactor_a=biogrid_id_interactor or
                    biogrid_id_interactor_b=biogrid_id_interactor
                where organism_interactor=?
            """, (taxid,))
        else:
            cur = self.db.execute("""\
                select biogrid_id_interactor_a, biogrid_id_interactor_a, score
                from links
            """)
        edges = cur.fetchall()
        return edges

    def edges(self, id):
        """
        Return a list of all interactions where id is a participant
        (a list of 3-tuples (id_a, id_b, score)).

        """

        cur = self.db.execute("""\
            select biogrid_id_interactor_a, biogrid_id_interactor_b, score
            from links
            where biogrid_id_interactor_a=? or biogrid_id_interactor_b=?
        """, (id, id))
        return cur.fetchall()

    def all_edges_annotated(self, taxid=None):
        """
        Return a list of all edges annotated. If taxid is not None
        return the edges for this organism only.

        """
        if taxid is not None:
            cur = self.db.execute("""\
                select *
                from links left join proteins on
                    biogrid_id_interactor_a=biogrid_id_interactor or
                    biogrid_id_interactor_b=biogrid_id_interactor
                where organism_interactor=?
            """, (taxid,))
        else:
            cur = self.db.execute("""\
                select *
                from links
            """)
        edges = cur.fetchall()
        return edges

    def edges_annotated(self, id):
        """ Return a list of all links
        """
        cur = self.db.execute("""\
            select *
            from links
            where biogrid_id_interactor_a=? or biogrid_id_interactor_b=?
        """, (id, id))
        return cur.fetchall()

    def search_id(self, name, taxid=None):
        """
        Search the database for protein name. Return a list of matching
        primary ids. Use `taxid` to limit the results to a single organism.

        """
        # TODO: synonyms_interactor can contain multiple synonyms
        # (should create an indexed table of synonyms)
        if taxid is None:
            cur = self.db.execute("""\
                select biogrid_id_interactor
                from proteins
                where (biogrid_id_interactor=? or
                       entrez_gene_interactor=? or
                       systematic_name_interactor=? or
                       official_symbol_interactor=? or
                       synonyms_interactor=?)
            """, ((name,) * 5))
        else:
            cur = self.db.execute("""\
                select biogrid_id_interactor
                from proteins
                where (biogrid_id_interactor=? or
                       entrez_gene_interactor=? or
                       systematic_name_interactor=? or
                       official_symbol_interactor=? or
                       synonyms_interactor=?)
                      and organism_interactor=?
            """, ((name,) * 5) + (taxid,))
        res = map(itemgetter(0), cur)
        return res

    @classmethod
    def download_data(cls, address):
        """
        Pass the address of the latest BIOGRID-ALL release (in tab2 format).
        """
        stream = urllib2.urlopen(address)
        stream = StringIO(stream.read())
        zfile = zipfile.ZipFile(stream)
        # Expecting only one file.
        filename = zfile.namelist()[0]

        filepath = serverfiles.localpath("PPI", "BIOGRID-ALL.tab2")
        mkdir_p(os.path.dirname(filepath))

        with open(filepath, "wb") as f:
            shutil.copyfileobj(zfile.open(filename, "r"), f)

        cls.init_db(filepath)

    @classmethod
    def init_db(cls, filepath):
        """
        Initialize the sqlite data base from a BIOGRID-ALL.*tab2.txt file
        format.

        """
        dirname = os.path.dirname(filepath)
        rows = csv.reader(open(filepath, "rb"), delimiter="\t")
        rows.next()  # read the header line

        con = sqlite3.connect(os.path.join(dirname, BioGRID.SERVER_FILE))
        con.execute("drop table if exists links")  # Drop old table
        con.execute("drop table if exists proteins")  # Drop old table

        con.execute("""\
            create table links (
                biogrid_interaction_id text,
                biogrid_id_interactor_a text,
                biogrid_id_interactor_b text,
                experimental_system text,
                experimental_system_type text,
                author text,
                pubmed_id text,
                throughput text,
                score real,
                modification text,
                phenotypes text,
                qualifications text,
                tags text,
                source_database text
            )""")

        con.execute("""\
            create table proteins (
                biogrid_id_interactor text,
                entrez_gene_interactor text,
                systematic_name_interactor text,
                official_symbol_interactor text,
                synonyms_interactor text,
                organism_interactor text
            )""")

        proteins = {}

        # Values that go in the links table
        link_indices = [0, 3, 4, 11, 12, 13, 14, 17, 18, 19, 20, 21, 22, 23]
        # Values that go in the proteins table
        interactor_a_indices = [3, 1, 5, 7, 9, 15]
        # Values that go in the proteins table
        interactor_b_indices = [4, 2, 6, 8, 10, 16]

        def to_none(val):
            return None if val == "-" else val

        def processlinks(rowiter):
            for row in rowiter:
                fields = map(to_none, row)
                yield [fields[i] for i in link_indices]

                interactor_a = [fields[i] for i in interactor_a_indices]
                interactor_b = [fields[i] for i in interactor_b_indices]
                proteins[interactor_a[0]] = interactor_a
                proteins[interactor_b[0]] = interactor_b

        con.executemany("""
            insert into links values (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)
            """, processlinks(rows))

        con.executemany("""\
            insert into proteins values (?, ?, ?, ?, ?, ?)
            """, proteins.itervalues())
        con.commit()
        con.close()

    def init_db_index(self):
        """
        Will create an indexes (if not already present) in the database
        for faster searching by primary ids.

        """
        self.db.execute("""\
        create index if not exists index_on_biogrid_id_interactor_a
           on links (biogrid_id_interactor_a)
        """)
        self.db.execute("""\
        create index if not exists index_on_biogrid_id_interactor_b
           on links (biogrid_id_interactor_b)
        """)
        self.db.execute("""\
        create index if not exists index_on_biogrid_id_interactor
           on proteins (biogrid_id_interactor)
        """)


STRINGInteraction = namedtuple(
    "STRINGInteraciton",
    ["protein_id1",
     "protein_id2",
     "combined_score",
     "mode",
     "action",
     "score"]
)


class STRING(PPIDatabase):
    """
    Access `STRING <http://www.string-db.org/>`_ PPI database.
    """

    DATABASE_SCHEMA = """\
    Database schema
    ---------------
    table `links`:
        - `protein_id1`: id (text)
        - `protein_id2`: id (text)
        - `score`: combined score (int)

    table `actions`:
        - `protein_id1`: id (text)
        - `protein_id2`: id (text)
        - `mode`: mode (text)
        - `action`: action type (text)
        - `score`: action score (int)

    table `proteins`:
        - `protein_id`: protein id in STRING (text) (in the form of {taxid}.{name})
        - `taxid`: organism taxid (text)

    table `aliases`:
        - `protein_id: id (text)
        - `alias`: protein alias (text)
        - `source`: protein alias source (text)

    """
    DOMAIN = "PPI"
    FILENAME = "string.protein.{taxid}.sqlite"
    VERSION = "3.0"

    # Mapping from taxonomy.common_taxids() to taxids in STRING.

    TAXID_MAP = {"352472": "44689",  # Dictyostelium discoideum
                 "562": None,
                 "2104": "272634",   # Mycoplasma pneumoniae M129
                 "4530": "39947",    # Oryza sativa Japonica Group
                 "4754": None,
                 "8355": None,
                 "4577": None,
                 "11103": None}

    def __init__(self, taxid=None, database=None):
        if taxid is not None and database is not None:
            raise ValueError("taxid and database parameters are exclusive.")

        self.db = None

        if taxid is None and database is not None:
            if isinstance(database, sqlite3.Connection):
                self.db = database
                self.filename = None
            else:
                self.filename = database
                self.db = sqlite3.connect(database)
        elif taxid is not None and database is None:
            self.filename = serverfiles.localpath_download(
                self.DOMAIN, self.FILENAME.format(taxid=taxid)
            )
        elif taxid is None and database is None:
            # Back compatibility
            self.filename = serverfiles.localpath_download(
                "PPI", "string-protein.sqlite")
        else:
            assert False, "Not reachable"

        if self.db is None:
            self.db = sqlite3.connect(self.filename)

    @classmethod
    def default_db_filename(cls, taxid):
        return serverfiles.localpath(
            cls.DOMAIN, cls.FILENAME.format(taxid=taxid))

    @classmethod
    def common_taxids(cls):
        common = taxonomy.common_taxids()
        common = [cls.TAXID_MAP.get(id, id) for id in common]
        return filter(None, common)

    def organisms(self):
        """
        Return all organism taxids contained in this database.
        """
        cur = self.db.execute("select distinct taxid from proteins")
        return [r[0] for r in cur.fetchall()]

    def ids(self, taxid=None):
        """
        Return a list of all protein ids. If `taxid` is not None limit
        the results to ids from this organism only.

        """
        if taxid is not None:
            cur = self.db.execute("""\
                select protein_id
                from proteins
                where taxid=?
                """, (taxid,))
        else:
            cur = self.db.execute("""\
                select protein_id
                from proteins
                """)
        return [r[0] for r in cur.fetchall()]

    def synonyms(self, id):
        """
        Return a list of synonyms for primary `id` as reported by STRING
        (proteins.aliases.{version}.txt file)
        """
        cur = self.db.execute("""\
            select alias
            from aliases
            where protein_id=?
            """, (id,))
        res = cur.fetchall()
        return [r[0] for r in res]

    def synonyms_with_source(self, id):
        """
        Return a list of synonyms for primary `id` along with its
        source as reported by STRING (proteins.aliases.{version}.txt file)

        """
        cur = self.db.execute("""\
            select alias, source
            from aliases
            where protein_id=?
            """, (id,))
        res = cur.fetchall()
        return [(syn, set(source.split(" "))) \
                for syn, source in res]

    def all_edges(self, taxid=None):
        """
        Return a list of all edges. If taxid is not None return the edges
        for this organism only.

        .. note:: This may take some time (and memory).

        """
        if taxid is not None:
            cur = self.db.execute("""\
                select links.protein_id1, links.protein_id2, score
                from links join proteins on
                    links.protein_id1=proteins.protein_id
                where taxid=?
                """, (taxid,))
        else:
            cur = self.db.execute("""\
                select protein_id1, protein_id1, score
                from links
                """)
        return cur.fetchall()

    def edges(self, id):
        """
        Return a list of all edges (a list of 3-tuples (id1, id2, score)).
        """
        cur = self.db.execute("""\
            select protein_id1, protein_id2, score
            from links
            where protein_id1=?
            """, (id,))
        return cur.fetchall()

    def all_edges_annotated(self, taxid=None):
        res = []
        for id in self.ids(taxid):
            res.extend(self.edges_annotated(id))
        return res

    def edges_annotated(self, id):
        cur = self.db.execute("""\
            select links.protein_id1, links.protein_id2, links.score,
                   actions.action, actions.mode, actions.score
            from links left join actions on
                   links.protein_id1=actions.protein_id1 and
                   links.protein_id2=actions.protein_id2
            where links.protein_id1=?
        """, (id,))
        return map(STRINGInteraction._make, cur.fetchall())

    def search_id(self, name, taxid=None):
        if taxid is None:
            cur = self.db.execute("""\
                select proteins.protein_id
                from proteins natural join aliases
                where aliases.alias=?
            """, (name,))
        else:
            cur = self.db.execute("""\
                select proteins.protein_id
                from proteins natural join aliases
                where aliases.alias=? and proteins.taxid=?
            """, (name, taxid))
        return map(itemgetter(0), cur)

    @classmethod
    def download_data(cls, version, taxids=None):
        """
        Download the  PPI data for local work (this may take some time).
        Pass the version of the  STRING release e.g. v9.1.
        """
        if taxids is None:
            taxids = cls.common_taxids()

        for taxid in taxids:
            cls.init_db(version, taxid)

    @classmethod
    def init_db(cls, version, taxid, cache_dir=None, dbfilename=None):
        if cache_dir is None:
            cache_dir = serverfiles.localpath(cls.DOMAIN)

        if dbfilename is None:
            dbfilename = cls.default_db_filename(taxid)

        pjoin = os.path.join

        base_url = "http://www.string-db.org/newstring_download/"

        def paths(flatfile):
            url = "{flatfile}.{version}/{taxid}.{flatfile}.{version}.txt.gz"
            url = url.format(flatfile=flatfile, version=version, taxid=taxid)
            return posixpath.basename(url), base_url + url

        def ffname(pattern):
            return pattern.format(taxid=taxid, version=version)

        links_filename, links_url = paths("protein.links")

        actions_filename, actions_url = paths("protein.actions")

        aliases_filename, aliases_url = paths("protein.aliases")

        def download(filename, url):
            with open(pjoin(cache_dir, filename + ".tmp"), "wb") as dest:
                wget(url, dst_obj=dest, progress=True)

            shutil.move(pjoin(cache_dir, filename + ".tmp"),
                        pjoin(cache_dir, filename))

        for fname, url in [(links_filename, links_url),
                           (actions_filename, actions_url),
                           (aliases_filename, aliases_url)]:
            if not os.path.exists(pjoin(cache_dir, fname)):
                download(fname, url)

        links_fileobj = open(pjoin(cache_dir, links_filename), "rb")
        actions_fileobj = open(pjoin(cache_dir, actions_filename), "rb")
        aliases_fileobj = open(pjoin(cache_dir, aliases_filename), "rb")

        links_file = gzip.GzipFile(fileobj=links_fileobj)
        actions_file = gzip.GzipFile(fileobj=actions_fileobj)
        aliases_file = gzip.GzipFile(fileobj=aliases_fileobj)

        progress = ConsoleProgressBar("Processing {}:".format(links_filename))
        progress(0.0)

        def st_size(filename):
            return os.stat(pjoin(cache_dir, filename)).st_size

        filesize = st_size(links_filename)

        con = sqlite3.connect(dbfilename)

        with con:
            cls.clear_db(con)

            links_file.readline()  # read the header line

            reader = csv.reader(links_file, delimiter=" ")

            def read_links(reader, progress):
                for i, (p1, p2, score) in enumerate(reader):
                    yield p1, p2, int(score)

                    if i % 100000 == 0:
                        # Update the progress every 100000 lines
                        progress(100.0 * links_fileobj.tell() / filesize)

            con.executemany("INSERT INTO links VALUES (?, ?, ?)",
                            read_links(reader, progress))

            progress.finish()

            def part(string, sep, part):
                return string.split(sep)[part]

            con.create_function("part", 3, part)
            con.execute("""
                INSERT INTO proteins
                SELECT protein_id1, part(protein_id1, '.', 0)
                FROM (SELECT DISTINCT(protein_id1)
                     FROM links
                     ORDER BY protein_id1)
            """)

            filesize = st_size(actions_filename)

            actions_file.readline()  # read header line

            progress = ConsoleProgressBar("Processing actions:")
            reader = csv.reader(actions_file, delimiter="\t")

            def read_actions(reader):
                for i, (p1, p2, mode, action, a_is_acting, score) in \
                        enumerate(reader):
                    yield p1, p2, mode, action, int(score)

                    if i % 10000 == 0:
                        progress(100.0 * actions_fileobj.tell() / filesize)

            con.executemany("INSERT INTO actions VALUES (?, ?, ?, ?, ?)",
                            read_actions(reader))

            progress.finish()

            filesize = st_size(aliases_filename)
            aliases_file.readline()  # read header line

            progress = ConsoleProgressBar("Processing aliases:")

            reader = csv.reader(aliases_file, delimiter="\t")

            def read_aliases(reader, progress):
                for i, (taxid, name, alias, source) in enumerate(reader):
                    yield (".".join([taxid, name]),
                           alias.decode("utf-8", errors="ignore"),
                           source.decode("utf-8", errors="ignore"))
                    if i % 10000 == 0:
                        progress(100.0 * aliases_fileobj.tell() / filesize)

            con.executemany("INSERT INTO aliases VALUES (?, ?, ?)",
                            read_aliases(reader, progress))

            progress.finish()

            print "Indexing the database"
            cls.create_db_index(con)

            con.executescript("""
                DROP TABLE IF EXISTS version;
                CREATE TABLE version (
                     string_version text,
                     api_version text
                );""")

            con.execute("""
                INSERT INTO version
                VALUES (?, ?)""", (version, cls.VERSION))

    @classmethod
    def clear_db(cls, dbcon):
        dbcon.executescript(textwrap.dedent("""
            DROP TABLE IF EXISTS links;
            DROP TABLE IF EXISTS proteins;
            DROP TABLE IF EXISTS actions;
            DROP TABLE IF EXISTS aliases;
        """))

        dbcon.executescript(textwrap.dedent("""
            CREATE TABLE links
                (protein_id1 TEXT, protein_id2 TEXT, score INT);

            CREATE TABLE proteins
                (protein_id TEXT, taxid TEXT);

            CREATE TABLE actions
                (protein_id1 TEXT, protein_id2 TEXT, mode TEXT,
                 action TEXT, score INT);

            CREATE TABLE aliases
                (protein_id TEXT, alias TEXT, source TEXT);
        """))

    @classmethod
    def create_db_index(cls, dbcon):
        dbcon.executescript(textwrap.dedent("""
            CREATE INDEX IF NOT EXISTS index_link_protein_id1
                ON links (protein_id1);

            CREATE INDEX IF NOT EXISTS index_action_protein_id1
                ON actions (protein_id1);

            CREATE INDEX IF NOT EXISTS index_proteins_id
                ON proteins (protein_id);

            CREATE INDEX IF NOT EXISTS index_taxids
                ON proteins (taxid);

            CREATE INDEX IF NOT EXISTS index_aliases_id
                ON aliases (protein_id);

            CREATE INDEX IF NOT EXISTS index_aliases_alias
                ON aliases (alias);
        """))


STRINGDetailedInteraction = namedtuple(
    "STRINGDetailedInteraction",
    ["protein_id1",
     "protein_id2",
     "combined_score",
     "mode",
     "action",
     "score",
     "neighborhood",
     "fusion",
     "cooccurence",
     "coexpression",
     "experimental",
     "database",
     "textmining"]
)


class STRINGDetailed(STRING):
    """
    Access `STRING <http://www.string-db.org/>`_ PPI database.
    This class also allows access to subscores per channel.

    .. note::
        This data is released under a `Creative Commons
        Attribution-Noncommercial-Share Alike 3.0 License
        <http://creativecommons.org/licenses/by-nc-sa/3.0/>`_.

        If you want to use this data for commercial purposes you must
        get a license from STRING.

    """

    DATABASE_SCHEMA = """\
    DATABASE SCHEMA
    ===============

    table `evidence`:
        - `protein_id1`: protein id (text)
        - `protein_id2`: protein id (text)
        - `neighborhood`: score (int)
        - `fusion`: score (int)
        - `cooccurence`: score (int)
        - `coexpression`: score (int)
        - `experimental`: score (int)
        - `database`: score (int)
        - `textmining`: score (int)

    """
    FILENAME_DETAILED = "string.protein.detailed.{taxid}.sqlite"

    def __init__(self, taxid=None, database=None, detailed_database=None):
        STRING.__init__(self, taxid, database)
        if taxid is not None and detailed_database is not None:
            raise ValueError("taxid and detailed_database are exclusive")

        db_file = serverfiles.localpath(self.DOMAIN, self.FILENAME)
        if taxid is not None and detailed_database is None:
            detailed_database = serverfiles.localpath_download(
                self.DOMAIN,
                self.FILENAME_DETAILED.format(taxid=taxid)
            )
        elif taxid is None and detailed_database is not None:
            detailed_database = detailed_database
        elif taxid is None and detailed_database is None:
            # Back compatibility
            detailed_database = serverfiles.localpath_download(
                "PPI", "string-protein-detailed.sqlite")

        self.db_detailed = sqlite3.connect(detailed_database)
        self.db_detailed.execute("ATTACH DATABASE ? as string", (db_file,))

    def edges_annotated(self, id):
        edges = STRING.edges_annotated(self, id)
        edges_nc = []
        for edge in edges:
            id1, id2 = edge.protein_id1, edge.protein_id2
            cur = self.db_detailed.execute("""
                SELECT neighborhood, fusion, cooccurence, coexpression,
                       experimental, database, textmining
                FROM evidence
                WHERE protein_id1=? AND protein_id2=?
                """, (id1, id2))
            res = cur.fetchone()
            if res:
                evidence = res
            else:
                evidence = [0] * 7
            edges_nc.append(
                STRINGDetailedInteraction(*(tuple(edge) + tuple(evidence)))
            )
        return edges_nc

    @classmethod
    def init_db(cls, version, taxid, cache_dir=None, dbfilename=None):
        if cache_dir is None:
            cache_dir = serverfiles.localpath(cls.DOMAIN)
        if dbfilename is None:
            dbfilename = serverfiles.localpath(
                cls.DOMAIN,
                "string-protein-detailed.{taxid}.sqlite".format(taxid=taxid)
            )

        pjoin = os.path.join

        base_url = "http://www.string-db.org/newstring_download/"
        filename = "{taxid}.protein.links.detailed.{version}.txt.gz"
        filename = filename.format(version=version, taxid=taxid)
        url = base_url + "protein.links.detailed.{version}/" + filename
        url = url.format(version=version)

        if not os.path.exists(pjoin(cache_dir, filename)):
            wget(url, cache_dir, progress=True)

        links_fileobj = open(pjoin(cache_dir, filename), "rb")
        links_file = gzip.GzipFile(fileobj=links_fileobj)

        con = sqlite3.connect(dbfilename)
        with con:
            con.execute("""
                DROP TABLE IF EXISTS evidence
            """)

            con.execute("""
                CREATE TABLE evidence(
                     protein_id1 TEXT,
                     protein_id2 TEXT,
                     neighborhood INTEGER,
                     fusion INTEGER,
                     cooccurence INTEGER,
                     coexpression INTEGER,
                     experimental INTEGER,
                     database INTEGER,
                     textmining INTEGER
                    )
                """)

            links = csv.reader(links_file, delimiter=" ")
            links.next()  # Read header
            filesize = os.stat(pjoin(cache_dir, filename)).st_size

            progress = ConsoleProgressBar("Processing links file:")
            progress(1.0)

            def read_links(reader):
                for i, (p1, p2, n, f, c, cx, ex, db, t, _) in \
                        enumerate(reader):
                    yield p1, p2, n, f, c, cx, ex, db, t

                    if i % 10000 == 0:
                        progress(100.0 * links_fileobj.tell() / filesize)

            con.executemany("""
                INSERT INTO evidence
                VALUES  (?, ?, ?, ?, ?, ?, ?, ?, ?)
                """, read_links(links))

            progress.finish()

            print "Indexing"
            con.execute("""\
                CREATE INDEX IF NOT EXISTS index_evidence
                    ON evidence (protein_id1, protein_id2)
            """)

            con.executescript("""
                DROP TABLE IF EXISTS version;

                CREATE TABLE version (
                     string_version text,
                     api_version text
                );
                """)

            con.execute("""
                INSERT INTO version
                VALUES (?, ?)""", (version, cls.VERSION))

    @classmethod
    def download_data(cls, version, taxids=None):
        if taxids is None:
            taxids = cls.common_taxids()

        for taxid in taxids:
            cls.init_db(version, taxid)


##########
# Obsolete
##########

class Interaction(object):
    def __init__(self, protein1, protein2, ref1=None, ref2=None, conf1=None, conf2=None):
        self.protein1, self.protein2 = protein1, protein2
        self.ref1, self.ref2 = ref1, ref2
        self.conf1, self.conf2 = conf1, conf2
        self.org1, self.org2 = None, None


class MIPS(object):
    VERSION = 1

    def __init__(self):
        self.load()

    def load(self):
        self.protein_names = defaultdict(set)
        self.refs = {}
        self.confidance = {}

        def process(element):
            d = {}
            participants = element.getElementsByTagName("proteinParticipant")
            proteins = []
            for protein in participants:
                interactor = protein.getElementsByTagName("proteinInteractor")[0]
                names = []
                for name in interactor.getElementsByTagName("shortLabel") + \
                            interactor.getElementsByTagName("fullName"):
                    names.append((name.tagName, name.childNodes[0].data))

                refs = []
                for ref in interactor.getElementsByTagName("primaryRef"):
                    refs += [(ref.tagName, ref.attributes.items())]
                org = dict(interactor.getElementsByTagName("organism")[0].attributes.items()).get("ncbiTaxId")
                conf = protein.getElementsByTagName("confidence")[0].attributes.items()
                proteins.append((names, refs, conf, org))
            interaction = Interaction(proteins[0][0][1][1], proteins[1][0][1][1])
            interaction.ref1, interaction.ref2 = proteins[0][1], proteins[1][1]
            interaction.conf1, interaction.conf2 = proteins[0][2], proteins[1][2]
            interaction.org1, interaction.org2 = proteins[0][3], proteins[1][3]

            self.protein_names[interaction.protein1].add(proteins[0][0][0][1])
            self.protein_names[interaction.protein2].add(proteins[1][0][0][1])

            return interaction

        document = minidom.parse(serverfiles.localpath_download("PPI", "allppis.xml"))
        interactions = document.getElementsByTagName("interaction")
        self.interactions = [process(interaction) for interaction in interactions]

        self.protein_interactions = defaultdict(set)

        for inter in self.interactions:
            self.protein_names[inter.protein1] = dict(inter.ref1[0][1]).get("id")
            self.protein_names[inter.protein2] = dict(inter.ref2[0][1]).get("id")
            self.protein_interactions[inter.protein1].add(inter)
            self.protein_interactions[inter.protein2].add(inter)

    def __iter__(self):
        return iter(self.interactions)

    @classmethod
    def download(cls):
        src = urllib2.urlopen("http://mips.helmholtz-muenchen.de/proj/ppi/data/mppi.gz")
        dest = serverfiles.localpath("PPI", "mppi.gz")
        shutil.copyfileobj(src, open(dest, "wb"))

    @classmethod
    @pickled_cache(None, [("PPI", "allppis.xml")], version=1)
    def _get_instance(cls):
        return MIPS()

    @classmethod
    def get_instance(cls):
        if not hasattr(cls, "_instance"):
            cls._instance = cls._get_instance()
        return cls._instance


def mips_interactions(protein=None):
    mips = MIPS.get_instance()
    if protein is None:
        return list(mips)
    else:
        return mips.protein_interactions.get(protein)


def mips_proteins():
    return set(MIPS.get_instance().protein_names.keys())


class BioGRIDInteraction(object):
    """ An object representing a BioGRID interaction. Each member of this object
    represents a data from a single column of BIOGRID-ALL.tab file.

    Attributes:
        - *interactor_a*    - BioGRID identifier
        - *interactor_b*    - BioGRID identifier
        - *official_symbol_a*    - An official symbol for *interactor_a*
        - *official_symbol_b*    - An official symbol for *interactor_b*
        - *aliases_for_a*    - Aliases separated by '|'
        - *aliases_for_b*    - Aliases separated by '|'
        - *experimental_system*     - Experimental system (see BioGRID documentation on www.thebiogrid.org for a list of valid entrys)
        - *source*    -
        - *organism_a_id*    - NCBI Taxonomy identifier for *interactor_a*'s organism
        - *organism_b_id*    - NCBI Taxonomy identifier for *interactor_b*'s organism
    """
    __slots__ = ["interactor_a", "interactor_b", "official_symbol_a", "official_symbol_b", "aliases_for_a", "aliases_for_b", "experimental_system", "source", "pubmed_id", "organism_a_id", "organism_b_id"]

    def __init__(self, line):
        for attr, val in zip(self.__slots__, line.split("\t")):
            setattr(self, attr, val)


class _BioGRID_Old(object):
    """ A BioGRID database interface
    Example::
        >>> ## finding all interactions for Homo sapiens sapiens
        >>> grid = BioGRID(case_insensitive=True)
        >>> proteins = proteins = biogrid.proteins() ## All proteins
        >>> proteins = [p for p in proteins if any(["9606" in [int.organism_a_id, int.organism_b_id] for int in grid.get(p)])]
    """
    VERSION = 1

    def __init__(self, case_insensitive=True):
#        warnings.warn("obiPPi._BioGRID_Old class is deprecated. Use obiPPI.BioGRID")
        self.case_insensitive = case_insensitive
        self._case = (lambda name: name.lower()) if self.case_insensitive else (lambda name: name)
        self.load()

    def load(self):
        text = open(serverfiles.localpath_download("PPI", "BIOGRID-ALL.tab"), "rb").read()
        text = text.split("SOURCE\tPUBMED_ID\tORGANISM_A_ID\tORGANISM_B_ID\n", 1)[-1]
        self.interactions = [BioGRIDInteraction(line) for line in text.splitlines() if line.strip()]

        self.protein_interactions = defaultdict(set)
        self.protein_names = {}

        case = self._case

        def update(keys, value, collection):
            for k in keys:
                collection.setdefault(k, set()).add(value)

        for inter in self.interactions:
            update(map(case, [inter.official_symbol_a] + inter.aliases_for_a.split("|")), case(inter.interactor_a), self.protein_names)
            update(map(case, [inter.official_symbol_b] + inter.aliases_for_b.split("|")), case(inter.interactor_b), self.protein_names)

            self.protein_interactions[case(inter.interactor_a)].add(inter)
            self.protein_interactions[case(inter.interactor_b)].add(inter)

        self.protein_interactions = dict(self.protein_interactions)

        if case("N/A") in self.protein_names:
            del self.protein_names[case("N/A")]

    def proteins(self):
        """ Return all protein names in BioGRID (from INTERACTOR_A, and INTERACTOR_B columns).
        """
        return self.protein_interactions.keys()

    def __iter__(self):
        """ Iterate over all BioGRIDInteraction objects.
        """
        return iter(self.interactions)

    def __getitem__(self, key):
        """ Return a list of protein interactions that a protein is a part of.
        """
        key = self._case(key)
#        keys = self.protein_alias_matcher.match(key)
        if key not in self.protein_interactions:
            keys = self.protein_names.get(key, [])
        else:
            keys = [key]
        if keys:
            return list(reduce(set.union, [self.protein_interactions.get(k, []) for k in keys], set()))
        else:
            raise KeyError(key)

    def get(self, key, default=None):
        """ Return a list of protein interactions that a protein is a part of.
        """
        key = self._case(key)
#        keys = self.protein_alias_matcher.match(key)
        if key not in self.protein_interactions:
            keys = self.protein_names.get(keys, [])
        else:
            keys = [key]
        if keys:
            return list(reduce(set.union, [self.protein_interactions.get(k, []) for k in keys], set()))
        else:
            return default

    @classmethod
    def get_instance(cls):
        if getattr(cls, "_instance", None) is None:
            cls._instance = _BioGRID_Old()
        return cls._instance


def biogrid_interactions(name=None):
    """Return a list of protein interactions (BioGRIDInteraction objects) that a protein is a part of.
    """
    if name:
        return list(_BioGRID_Old.get_instance().get(name, set()))
    else:
        return _BioGRID_Old.get_instance().interactions


def biogrid_proteins():
    """ Return all protein names in BioGRID (from INTERACTOR_A, and INTERACTOR_B columns).
    """
    return _BioGRID_Old.get_instance().proteins()


if __name__ == "__main__":
    for protein in mips_proteins():
        print "Protein", protein, "interacts with",
        print ",".join(set(reduce(list.__add__, [[inter.protein1, inter.protein2] for inter in mips_interactions(protein)], [])) - set([protein]))

